@@ -89,83 +89,188 @@ static inline void arr_store_rowcol(
 #endif
 
 parasail_result_t* FNAME(
-        const char * const restrict _s1, const int s1Len,
+        const char * const restrict _s1, const int _s1Len,
         const char * const restrict _s2, const int s2Len,
         const int open, const int gap, const parasail_matrix_t *matrix)
 {
-    const int32_t N = 4; /* number of values in vector */
-    const int32_t PAD = N-1;
-    const int32_t PAD2 = PAD*2;
-    const int32_t s1Len_PAD = s1Len+PAD;
-    const int32_t s2Len_PAD = s2Len+PAD;
-    int32_t * const restrict s1      = parasail_memalign_int32_t(16, s1Len+PAD);
-    int32_t * const restrict s2B     = parasail_memalign_int32_t(16, s2Len+PAD2);
-    int32_t * const restrict _H_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
-    int32_t * const restrict _HM_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
-    int32_t * const restrict _HS_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
-    int32_t * const restrict _HL_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
-    int32_t * const restrict _F_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
-    int32_t * const restrict _FM_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
-    int32_t * const restrict _FS_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
-    int32_t * const restrict _FL_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
-    int32_t * const restrict s2 = s2B+PAD; /* will allow later for negative indices */
-    int32_t * const restrict H_pr = _H_pr+PAD;
-    int32_t * const restrict HM_pr = _HM_pr+PAD;
-    int32_t * const restrict HS_pr = _HS_pr+PAD;
-    int32_t * const restrict HL_pr = _HL_pr+PAD;
-    int32_t * const restrict F_pr = _F_pr+PAD;
-    int32_t * const restrict FM_pr = _FM_pr+PAD;
-    int32_t * const restrict FS_pr = _FS_pr+PAD;
-    int32_t * const restrict FL_pr = _FL_pr+PAD;
-#ifdef PARASAIL_TABLE
-    parasail_result_t *result = parasail_result_new_table3(s1Len, s2Len);
-#else
-#ifdef PARASAIL_ROWCOL
-    parasail_result_t *result = parasail_result_new_rowcol3(s1Len, s2Len);
-#else
-    parasail_result_t *result = parasail_result_new_stats();
-#endif
-#endif
+    /* declare local variables */
+    int32_t N = 0;
+    int32_t PAD = 0;
+    int32_t PAD2 = 0;
+    int32_t s1Len = 0;
+    int32_t s1Len_PAD = 0;
+    int32_t s2Len_PAD = 0;
+    int32_t * restrict s1 = NULL;
+    int32_t * restrict s2B = NULL;
+    int32_t * restrict _H_pr = NULL;
+    int32_t * restrict _HM_pr = NULL;
+    int32_t * restrict _HS_pr = NULL;
+    int32_t * restrict _HL_pr = NULL;
+    int32_t * restrict _F_pr = NULL;
+    int32_t * restrict _FM_pr = NULL;
+    int32_t * restrict _FS_pr = NULL;
+    int32_t * restrict _FL_pr = NULL;
+    int32_t * restrict s2 = NULL;
+    int32_t * restrict H_pr = NULL;
+    int32_t * restrict HM_pr = NULL;
+    int32_t * restrict HS_pr = NULL;
+    int32_t * restrict HL_pr = NULL;
+    int32_t * restrict F_pr = NULL;
+    int32_t * restrict FM_pr = NULL;
+    int32_t * restrict FS_pr = NULL;
+    int32_t * restrict FL_pr = NULL;
+    parasail_result_t *result = NULL;
     int32_t i = 0;
     int32_t j = 0;
-    int32_t end_query = s1Len-1;
-    int32_t end_ref = s2Len-1;
-    const int32_t NEG_LIMIT = (-open < matrix->min ?
-        INT32_MIN + open : INT32_MIN - matrix->min) + 1;
-    const int32_t POS_LIMIT = INT32_MAX - matrix->max - 1;
-    int32_t score = NEG_LIMIT;
-    int32_t matches = NEG_LIMIT;
-    int32_t similar = NEG_LIMIT;
-    int32_t length = NEG_LIMIT;
-    vec128i vNegLimit = _mm_set1_epi32(NEG_LIMIT);
-    vec128i vPosLimit = _mm_set1_epi32(POS_LIMIT);
-    vec128i vSaturationCheckMin = vPosLimit;
-    vec128i vSaturationCheckMax = vNegLimit;
-    vec128i vNegInf = _mm_set1_epi32(NEG_LIMIT);
-    vec128i vOpen = _mm_set1_epi32(open);
-    vec128i vGap  = _mm_set1_epi32(gap);
-    vec128i vZero = _mm_set1_epi32(0);
-    vec128i vNegInf0 = _mm_insert_epi32(vZero, NEG_LIMIT, 3);
-    vec128i vOne = _mm_set1_epi32(1);
-    vec128i vN = _mm_set1_epi32(N);
-    vec128i vGapN = _mm_set1_epi32(gap*N);
-    vec128i vNegOne = _mm_set1_epi32(-1);
-    vec128i vI = _mm_set_epi32(0,1,2,3);
-    vec128i vJreset = _mm_set_epi32(0,-1,-2,-3);
-    vec128i vMaxH = vNegInf;
-    vec128i vMaxM = vNegInf;
-    vec128i vMaxS = vNegInf;
-    vec128i vMaxL = vNegInf;
-    vec128i vILimit = _mm_set1_epi32(s1Len);
-    vec128i vILimit1 = _mm_sub_epi32(vILimit, vOne);
-    vec128i vJLimit = _mm_set1_epi32(s2Len);
-    vec128i vJLimit1 = _mm_sub_epi32(vJLimit, vOne);
-    vec128i vIBoundary = _mm_set_epi32(
+    int32_t end_query = 0;
+    int32_t end_ref = 0;
+    int32_t NEG_LIMIT = 0;
+    int32_t POS_LIMIT = 0;
+    int32_t score = 0;
+    int32_t matches = 0;
+    int32_t similar = 0;
+    int32_t length = 0;
+    vec128i vNegLimit;
+    vec128i vPosLimit;
+    vec128i vSaturationCheckMin;
+    vec128i vSaturationCheckMax;
+    vec128i vNegInf;
+    vec128i vOpen;
+    vec128i vGap;
+    vec128i vZero;
+    vec128i vNegInf0;
+    vec128i vOne;
+    vec128i vN;
+    vec128i vGapN;
+    vec128i vNegOne;
+    vec128i vI;
+    vec128i vJreset;
+    vec128i vMaxH;
+    vec128i vMaxM;
+    vec128i vMaxS;
+    vec128i vMaxL;
+    vec128i vILimit;
+    vec128i vILimit1;
+    vec128i vJLimit;
+    vec128i vJLimit1;
+    vec128i vIBoundary;
+
+    /* validate inputs */
+    PARASAIL_CHECK_NULL(_s2);
+    PARASAIL_CHECK_GT0(s2Len);
+    PARASAIL_CHECK_GE0(open);
+    PARASAIL_CHECK_GE0(gap);
+    PARASAIL_CHECK_NULL(matrix);
+    if (matrix->type == PARASAIL_MATRIX_TYPE_PSSM) {
+        PARASAIL_CHECK_NULL_PSSM_STATS(_s1);
+    }
+    else {
+        PARASAIL_CHECK_NULL(_s1);
+        PARASAIL_CHECK_GT0(_s1Len);
+    }
+
+    /* initialize stack variables */
+    N = 4; /* number of values in vector */
+    PAD = N-1;
+    PAD2 = PAD*2;
+    s1Len = matrix->type == PARASAIL_MATRIX_TYPE_SQUARE ? _s1Len : matrix->length;
+    s1Len_PAD = s1Len+PAD;
+    s2Len_PAD = s2Len+PAD;
+    i = 0;
+    j = 0;
+    end_query = s1Len-1;
+    end_ref = s2Len-1;
+    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
+    POS_LIMIT = INT32_MAX - matrix->max - 1;
+    score = NEG_LIMIT;
+    matches = NEG_LIMIT;
+    similar = NEG_LIMIT;
+    length = NEG_LIMIT;
+    vNegLimit = _mm_set1_epi32(NEG_LIMIT);
+    vPosLimit = _mm_set1_epi32(POS_LIMIT);
+    vSaturationCheckMin = vPosLimit;
+    vSaturationCheckMax = vNegLimit;
+    vNegInf = _mm_set1_epi32(NEG_LIMIT);
+    vOpen = _mm_set1_epi32(open);
+    vGap  = _mm_set1_epi32(gap);
+    vZero = _mm_set1_epi32(0);
+    vNegInf0 = _mm_insert_epi32(vZero, NEG_LIMIT, 3);
+    vOne = _mm_set1_epi32(1);
+    vN = _mm_set1_epi32(N);
+    vGapN = _mm_set1_epi32(gap*N);
+    vNegOne = _mm_set1_epi32(-1);
+    vI = _mm_set_epi32(0,1,2,3);
+    vJreset = _mm_set_epi32(0,-1,-2,-3);
+    vMaxH = vNegInf;
+    vMaxM = vNegInf;
+    vMaxS = vNegInf;
+    vMaxL = vNegInf;
+    vILimit = _mm_set1_epi32(s1Len);
+    vILimit1 = _mm_sub_epi32(vILimit, vOne);
+    vJLimit = _mm_set1_epi32(s2Len);
+    vJLimit1 = _mm_sub_epi32(vJLimit, vOne);
+    vIBoundary = _mm_set_epi32(
             -open-0*gap,
             -open-1*gap,
             -open-2*gap,
             -open-3*gap);
 
+    /* initialize result */
+#ifdef PARASAIL_TABLE
+    result = parasail_result_new_table3(s1Len, s2Len);
+#else
+#ifdef PARASAIL_ROWCOL
+    result = parasail_result_new_rowcol3(s1Len, s2Len);
+#else
+    result = parasail_result_new_stats();
+#endif
+#endif
+    if (!result) return NULL;
+
+    /* set known flags */
+    result->flag |= PARASAIL_FLAG_NW | PARASAIL_FLAG_DIAG
+        | PARASAIL_FLAG_STATS
+        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_4;
+#ifdef PARASAIL_TABLE
+    result->flag |= PARASAIL_FLAG_TABLE;
+#endif
+#ifdef PARASAIL_ROWCOL
+    result->flag |= PARASAIL_FLAG_ROWCOL;
+#endif
+
+    /* initialize heap variables */
+    s1     = parasail_memalign_int32_t(16, s1Len+PAD);
+    s2B    = parasail_memalign_int32_t(16, s2Len+PAD2);
+    _H_pr  = parasail_memalign_int32_t(16, s2Len+PAD2);
+    _HM_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
+    _HS_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
+    _HL_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
+    _F_pr  = parasail_memalign_int32_t(16, s2Len+PAD2);
+    _FM_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
+    _FS_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
+    _FL_pr = parasail_memalign_int32_t(16, s2Len+PAD2);
+    s2 = s2B+PAD; /* will allow later for negative indices */
+    H_pr = _H_pr+PAD;
+    HM_pr = _HM_pr+PAD;
+    HS_pr = _HS_pr+PAD;
+    HL_pr = _HL_pr+PAD;
+    F_pr = _F_pr+PAD;
+    FM_pr = _FM_pr+PAD;
+    FS_pr = _FS_pr+PAD;
+    FL_pr = _FL_pr+PAD;
+
+    /* validate heap variables */
+    if (!s1) return NULL;
+    if (!s2B) return NULL;
+    if (!_H_pr) return NULL;
+    if (!_HM_pr) return NULL;
+    if (!_HS_pr) return NULL;
+    if (!_HL_pr) return NULL;
+    if (!_F_pr) return NULL;
+    if (!_FM_pr) return NULL;
+    if (!_FS_pr) return NULL;
+    if (!_FL_pr) return NULL;
+
     /* convert _s1 from char to int in range 0-23 */
     for (i=0; i<s1Len; ++i) {
         s1[i] = matrix->mapper[(unsigned char)_s1[i]];
@@ -254,10 +359,10 @@ parasail_result_t* FNAME(
                 s1[i+2],
                 s1[i+3]);
         vec128i vs2 = vNegInf;
-        const int * const restrict matrow0 = &matrix->matrix[matrix->size*s1[i+0]];
-        const int * const restrict matrow1 = &matrix->matrix[matrix->size*s1[i+1]];
-        const int * const restrict matrow2 = &matrix->matrix[matrix->size*s1[i+2]];
-        const int * const restrict matrow3 = &matrix->matrix[matrix->size*s1[i+3]];
+        const int * const restrict matrow0 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+0] : ((i+0 >= s1Len) ? s1Len-1 : i+0))];
+        const int * const restrict matrow1 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+1] : ((i+1 >= s1Len) ? s1Len-1 : i+1))];
+        const int * const restrict matrow2 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+2] : ((i+2 >= s1Len) ? s1Len-1 : i+2))];
+        const int * const restrict matrow3 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+3] : ((i+3 >= s1Len) ? s1Len-1 : i+3))];
         vNH = _mm_insert_epi32(vNH, H_pr[-1], 3);
         vWH = _mm_insert_epi32(vWH, -open - i*gap, 3);
         H_pr[-1] = -open - (i+N)*gap;
@@ -343,13 +448,15 @@ parasail_result_t* FNAME(
                 vES = _mm_andnot_si128(cond, vES);
                 vEL = _mm_andnot_si128(cond, vEL);
             }
-            vSaturationCheckMin = _mm_min_epi32(vSaturationCheckMin, vWH);
-            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vWH);
-            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vWM);
-            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vWS);
-            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vWL);
-            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vWL);
-            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vJ);
+            /* cannot start checking sat until after J clears boundary */
+            if (j > PAD) {
+                vSaturationCheckMin = _mm_min_epi32(vSaturationCheckMin, vWH);
+                vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vWH);
+                vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vWM);
+                vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vWS);
+                vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vWL);
+                vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vWL);
+            }
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->stats->tables->score_table, vWH, i, s1Len, j, s2Len);
             arr_store_si128(result->stats->tables->matches_table, vWM, i, s1Len, j, s2Len);
@@ -384,7 +491,6 @@ parasail_result_t* FNAME(
             vJ = _mm_add_epi32(vJ, vOne);
         }
         vI = _mm_add_epi32(vI, vN);
-        vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vI);
         vIBoundary = _mm_sub_epi32(vIBoundary, vGapN);
     }
 
@@ -422,15 +528,6 @@ parasail_result_t* FNAME(
     result->stats->matches = matches;
     result->stats->similar = similar;
     result->stats->length = length;
-    result->flag |= PARASAIL_FLAG_NW | PARASAIL_FLAG_DIAG
-        | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_4;
-#ifdef PARASAIL_TABLE
-    result->flag |= PARASAIL_FLAG_TABLE;
-#endif
-#ifdef PARASAIL_ROWCOL
-    result->flag |= PARASAIL_FLAG_ROWCOL;
-#endif
 
     parasail_free(_FL_pr);
     parasail_free(_FS_pr);
