@@ -13,7 +13,6 @@
 #include <intrin.h>
 #else
 #include <emmintrin.h>
-#include <smmintrin.h>
 #endif
 
 #include "parasail.h"
@@ -21,6 +20,22 @@
 #include "parasail/internal_sse.h"
 
 
+static inline __m128i _mm_cmpeq_epi64_rpl(__m128i a, __m128i b) {
+    __m128i_64_t A;
+    __m128i_64_t B;
+    A.m = a;
+    B.m = b;
+    A.v[0] = (A.v[0]==B.v[0]) ? 0xFFFFFFFFFFFFFFFF : 0;
+    A.v[1] = (A.v[1]==B.v[1]) ? 0xFFFFFFFFFFFFFFFF : 0;
+    return A.m;
+}
+
+static inline __m128i _mm_blendv_epi8_rpl(__m128i a, __m128i b, __m128i mask) {
+    a = _mm_andnot_si128(mask, a);
+    a = _mm_or_si128(a, _mm_and_si128(mask, b));
+    return a;
+}
+
 static inline __m128i _mm_cmpgt_epi64_rpl(__m128i a, __m128i b) {
     __m128i_64_t A;
     __m128i_64_t B;
@@ -31,16 +46,12 @@ static inline __m128i _mm_cmpgt_epi64_rpl(__m128i a, __m128i b) {
     return A.m;
 }
 
-#if HAVE_SSE41_MM_INSERT_EPI64
-#define _mm_insert_epi64_rpl _mm_insert_epi64
-#else
-static inline __m128i _mm_insert_epi64_rpl(__m128i a, int64_t i, int imm) {
+static inline __m128i _mm_insert_epi64_rpl(__m128i a, int64_t i, const int imm) {
     __m128i_64_t A;
     A.m = a;
     A.v[imm] = i;
     return A.m;
 }
-#endif
 
 static inline __m128i _mm_max_epi64_rpl(__m128i a, __m128i b) {
     __m128i_64_t A;
@@ -63,15 +74,11 @@ static inline __m128i _mm_set_epi64x_rpl(int64_t e1, int64_t e0) {
 }
 #endif
 
-#if HAVE_SSE41_MM_EXTRACT_EPI64
-#define _mm_extract_epi64_rpl _mm_extract_epi64
-#else
-static inline int64_t _mm_extract_epi64_rpl(__m128i a, int imm) {
+static inline int64_t _mm_extract_epi64_rpl(__m128i a, const int imm) {
     __m128i_64_t A;
     A.m = a;
     return A.v[imm];
 }
-#endif
 
 static inline __m128i _mm_min_epi64_rpl(__m128i a, __m128i b) {
     __m128i_64_t A;
@@ -149,91 +156,196 @@ static inline void arr_store_rowcol(
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_nw_stats_table_diag_sse41_128_64
+#define FNAME parasail_nw_stats_table_diag_sse2_128_64
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_nw_stats_rowcol_diag_sse41_128_64
+#define FNAME parasail_nw_stats_rowcol_diag_sse2_128_64
 #else
-#define FNAME parasail_nw_stats_diag_sse41_128_64
+#define FNAME parasail_nw_stats_diag_sse2_128_64
 #endif
 #endif
 
 parasail_result_t* FNAME(
-        const char * const restrict _s1, const int s1Len,
+        const char * const restrict _s1, const int _s1Len,
         const char * const restrict _s2, const int s2Len,
         const int open, const int gap, const parasail_matrix_t *matrix)
 {
-    const int32_t N = 2; /* number of values in vector */
-    const int32_t PAD = N-1;
-    const int32_t PAD2 = PAD*2;
-    const int32_t s1Len_PAD = s1Len+PAD;
-    const int32_t s2Len_PAD = s2Len+PAD;
-    int64_t * const restrict s1      = parasail_memalign_int64_t(16, s1Len+PAD);
-    int64_t * const restrict s2B     = parasail_memalign_int64_t(16, s2Len+PAD2);
-    int64_t * const restrict _H_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
-    int64_t * const restrict _HM_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
-    int64_t * const restrict _HS_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
-    int64_t * const restrict _HL_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
-    int64_t * const restrict _F_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
-    int64_t * const restrict _FM_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
-    int64_t * const restrict _FS_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
-    int64_t * const restrict _FL_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
-    int64_t * const restrict s2 = s2B+PAD; /* will allow later for negative indices */
-    int64_t * const restrict H_pr = _H_pr+PAD;
-    int64_t * const restrict HM_pr = _HM_pr+PAD;
-    int64_t * const restrict HS_pr = _HS_pr+PAD;
-    int64_t * const restrict HL_pr = _HL_pr+PAD;
-    int64_t * const restrict F_pr = _F_pr+PAD;
-    int64_t * const restrict FM_pr = _FM_pr+PAD;
-    int64_t * const restrict FS_pr = _FS_pr+PAD;
-    int64_t * const restrict FL_pr = _FL_pr+PAD;
-#ifdef PARASAIL_TABLE
-    parasail_result_t *result = parasail_result_new_table3(s1Len, s2Len);
-#else
-#ifdef PARASAIL_ROWCOL
-    parasail_result_t *result = parasail_result_new_rowcol3(s1Len, s2Len);
-#else
-    parasail_result_t *result = parasail_result_new_stats();
-#endif
-#endif
+    /* declare local variables */
+    int32_t N = 0;
+    int32_t PAD = 0;
+    int32_t PAD2 = 0;
+    int32_t s1Len = 0;
+    int32_t s1Len_PAD = 0;
+    int32_t s2Len_PAD = 0;
+    int64_t * restrict s1 = NULL;
+    int64_t * restrict s2B = NULL;
+    int64_t * restrict _H_pr = NULL;
+    int64_t * restrict _HM_pr = NULL;
+    int64_t * restrict _HS_pr = NULL;
+    int64_t * restrict _HL_pr = NULL;
+    int64_t * restrict _F_pr = NULL;
+    int64_t * restrict _FM_pr = NULL;
+    int64_t * restrict _FS_pr = NULL;
+    int64_t * restrict _FL_pr = NULL;
+    int64_t * restrict s2 = NULL;
+    int64_t * restrict H_pr = NULL;
+    int64_t * restrict HM_pr = NULL;
+    int64_t * restrict HS_pr = NULL;
+    int64_t * restrict HL_pr = NULL;
+    int64_t * restrict F_pr = NULL;
+    int64_t * restrict FM_pr = NULL;
+    int64_t * restrict FS_pr = NULL;
+    int64_t * restrict FL_pr = NULL;
+    parasail_result_t *result = NULL;
     int32_t i = 0;
     int32_t j = 0;
-    int32_t end_query = s1Len-1;
-    int32_t end_ref = s2Len-1;
-    const int64_t NEG_LIMIT = (-open < matrix->min ?
-        INT64_MIN + open : INT64_MIN - matrix->min) + 1;
-    const int64_t POS_LIMIT = INT64_MAX - matrix->max - 1;
-    int64_t score = NEG_LIMIT;
-    int64_t matches = NEG_LIMIT;
-    int64_t similar = NEG_LIMIT;
-    int64_t length = NEG_LIMIT;
-    __m128i vNegLimit = _mm_set1_epi64x_rpl(NEG_LIMIT);
-    __m128i vPosLimit = _mm_set1_epi64x_rpl(POS_LIMIT);
-    __m128i vSaturationCheckMin = vPosLimit;
-    __m128i vSaturationCheckMax = vNegLimit;
-    __m128i vNegInf = _mm_set1_epi64x_rpl(NEG_LIMIT);
-    __m128i vOpen = _mm_set1_epi64x_rpl(open);
-    __m128i vGap  = _mm_set1_epi64x_rpl(gap);
-    __m128i vZero = _mm_set1_epi64x_rpl(0);
-    __m128i vNegInf0 = _mm_insert_epi64_rpl(vZero, NEG_LIMIT, 1);
-    __m128i vOne = _mm_set1_epi64x_rpl(1);
-    __m128i vN = _mm_set1_epi64x_rpl(N);
-    __m128i vGapN = _mm_set1_epi64x_rpl(gap*N);
-    __m128i vNegOne = _mm_set1_epi64x_rpl(-1);
-    __m128i vI = _mm_set_epi64x_rpl(0,1);
-    __m128i vJreset = _mm_set_epi64x_rpl(0,-1);
-    __m128i vMaxH = vNegInf;
-    __m128i vMaxM = vNegInf;
-    __m128i vMaxS = vNegInf;
-    __m128i vMaxL = vNegInf;
-    __m128i vILimit = _mm_set1_epi64x_rpl(s1Len);
-    __m128i vILimit1 = _mm_sub_epi64(vILimit, vOne);
-    __m128i vJLimit = _mm_set1_epi64x_rpl(s2Len);
-    __m128i vJLimit1 = _mm_sub_epi64(vJLimit, vOne);
-    __m128i vIBoundary = _mm_set_epi64x_rpl(
+    int32_t end_query = 0;
+    int32_t end_ref = 0;
+    int64_t NEG_LIMIT = 0;
+    int64_t POS_LIMIT = 0;
+    int64_t score = 0;
+    int64_t matches = 0;
+    int64_t similar = 0;
+    int64_t length = 0;
+    __m128i vNegLimit;
+    __m128i vPosLimit;
+    __m128i vSaturationCheckMin;
+    __m128i vSaturationCheckMax;
+    __m128i vNegInf;
+    __m128i vOpen;
+    __m128i vGap;
+    __m128i vZero;
+    __m128i vNegInf0;
+    __m128i vOne;
+    __m128i vN;
+    __m128i vGapN;
+    __m128i vNegOne;
+    __m128i vI;
+    __m128i vJreset;
+    __m128i vMaxH;
+    __m128i vMaxM;
+    __m128i vMaxS;
+    __m128i vMaxL;
+    __m128i vILimit;
+    __m128i vILimit1;
+    __m128i vJLimit;
+    __m128i vJLimit1;
+    __m128i vIBoundary;
+
+    /* validate inputs */
+    PARASAIL_CHECK_NULL(_s2);
+    PARASAIL_CHECK_GT0(s2Len);
+    PARASAIL_CHECK_GE0(open);
+    PARASAIL_CHECK_GE0(gap);
+    PARASAIL_CHECK_NULL(matrix);
+    if (matrix->type == PARASAIL_MATRIX_TYPE_PSSM) {
+        PARASAIL_CHECK_NULL_PSSM_STATS(_s1);
+    }
+    else {
+        PARASAIL_CHECK_NULL(_s1);
+        PARASAIL_CHECK_GT0(_s1Len);
+    }
+
+    /* initialize stack variables */
+    N = 2; /* number of values in vector */
+    PAD = N-1;
+    PAD2 = PAD*2;
+    s1Len = matrix->type == PARASAIL_MATRIX_TYPE_SQUARE ? _s1Len : matrix->length;
+    s1Len_PAD = s1Len+PAD;
+    s2Len_PAD = s2Len+PAD;
+    i = 0;
+    j = 0;
+    end_query = s1Len-1;
+    end_ref = s2Len-1;
+    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
+    POS_LIMIT = INT64_MAX - matrix->max - 1;
+    score = NEG_LIMIT;
+    matches = NEG_LIMIT;
+    similar = NEG_LIMIT;
+    length = NEG_LIMIT;
+    vNegLimit = _mm_set1_epi64x_rpl(NEG_LIMIT);
+    vPosLimit = _mm_set1_epi64x_rpl(POS_LIMIT);
+    vSaturationCheckMin = vPosLimit;
+    vSaturationCheckMax = vNegLimit;
+    vNegInf = _mm_set1_epi64x_rpl(NEG_LIMIT);
+    vOpen = _mm_set1_epi64x_rpl(open);
+    vGap  = _mm_set1_epi64x_rpl(gap);
+    vZero = _mm_set1_epi64x_rpl(0);
+    vNegInf0 = _mm_insert_epi64_rpl(vZero, NEG_LIMIT, 1);
+    vOne = _mm_set1_epi64x_rpl(1);
+    vN = _mm_set1_epi64x_rpl(N);
+    vGapN = _mm_set1_epi64x_rpl(gap*N);
+    vNegOne = _mm_set1_epi64x_rpl(-1);
+    vI = _mm_set_epi64x_rpl(0,1);
+    vJreset = _mm_set_epi64x_rpl(0,-1);
+    vMaxH = vNegInf;
+    vMaxM = vNegInf;
+    vMaxS = vNegInf;
+    vMaxL = vNegInf;
+    vILimit = _mm_set1_epi64x_rpl(s1Len);
+    vILimit1 = _mm_sub_epi64(vILimit, vOne);
+    vJLimit = _mm_set1_epi64x_rpl(s2Len);
+    vJLimit1 = _mm_sub_epi64(vJLimit, vOne);
+    vIBoundary = _mm_set_epi64x_rpl(
             -open-0*gap,
             -open-1*gap);
 
+    /* initialize result */
+#ifdef PARASAIL_TABLE
+    result = parasail_result_new_table3(s1Len, s2Len);
+#else
+#ifdef PARASAIL_ROWCOL
+    result = parasail_result_new_rowcol3(s1Len, s2Len);
+#else
+    result = parasail_result_new_stats();
+#endif
+#endif
+    if (!result) return NULL;
+
+    /* set known flags */
+    result->flag |= PARASAIL_FLAG_NW | PARASAIL_FLAG_DIAG
+        | PARASAIL_FLAG_STATS
+        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_2;
+#ifdef PARASAIL_TABLE
+    result->flag |= PARASAIL_FLAG_TABLE;
+#endif
+#ifdef PARASAIL_ROWCOL
+    result->flag |= PARASAIL_FLAG_ROWCOL;
+#endif
+
+    /* initialize heap variables */
+    s1     = parasail_memalign_int64_t(16, s1Len+PAD);
+    s2B    = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _H_pr  = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _HM_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _HS_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _HL_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _F_pr  = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _FM_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _FS_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
+    _FL_pr = parasail_memalign_int64_t(16, s2Len+PAD2);
+    s2 = s2B+PAD; /* will allow later for negative indices */
+    H_pr = _H_pr+PAD;
+    HM_pr = _HM_pr+PAD;
+    HS_pr = _HS_pr+PAD;
+    HL_pr = _HL_pr+PAD;
+    F_pr = _F_pr+PAD;
+    FM_pr = _FM_pr+PAD;
+    FS_pr = _FS_pr+PAD;
+    FL_pr = _FL_pr+PAD;
+
+    /* validate heap variables */
+    if (!s1) return NULL;
+    if (!s2B) return NULL;
+    if (!_H_pr) return NULL;
+    if (!_HM_pr) return NULL;
+    if (!_HS_pr) return NULL;
+    if (!_HL_pr) return NULL;
+    if (!_F_pr) return NULL;
+    if (!_FM_pr) return NULL;
+    if (!_FS_pr) return NULL;
+    if (!_FL_pr) return NULL;
+
     /* convert _s1 from char to int in range 0-23 */
     for (i=0; i<s1Len; ++i) {
         s1[i] = matrix->mapper[(unsigned char)_s1[i]];
@@ -320,8 +432,8 @@ parasail_result_t* FNAME(
                 s1[i+0],
                 s1[i+1]);
         __m128i vs2 = vNegInf;
-        const int * const restrict matrow0 = &matrix->matrix[matrix->size*s1[i+0]];
-        const int * const restrict matrow1 = &matrix->matrix[matrix->size*s1[i+1]];
+        const int * const restrict matrow0 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+0] : ((i+0 >= s1Len) ? s1Len-1 : i+0))];
+        const int * const restrict matrow1 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+1] : ((i+1 >= s1Len) ? s1Len-1 : i+1))];
         vNH = _mm_insert_epi64_rpl(vNH, H_pr[-1], 1);
         vWH = _mm_insert_epi64_rpl(vWH, -open - i*gap, 1);
         H_pr[-1] = -open - (i+N)*gap;
@@ -352,17 +464,17 @@ parasail_result_t* FNAME(
             vF_ext = _mm_sub_epi64(vF, vGap);
             vF = _mm_max_epi64_rpl(vF_opn, vF_ext);
             case1 = _mm_cmpgt_epi64_rpl(vF_opn, vF_ext);
-            vFM = _mm_blendv_epi8(vFM, vNM, case1);
-            vFS = _mm_blendv_epi8(vFS, vNS, case1);
-            vFL = _mm_blendv_epi8(vFL, vNL, case1);
+            vFM = _mm_blendv_epi8_rpl(vFM, vNM, case1);
+            vFS = _mm_blendv_epi8_rpl(vFS, vNS, case1);
+            vFL = _mm_blendv_epi8_rpl(vFL, vNL, case1);
             vFL = _mm_add_epi64(vFL, vOne);
             vE_opn = _mm_sub_epi64(vWH, vOpen);
             vE_ext = _mm_sub_epi64(vE, vGap);
             vE = _mm_max_epi64_rpl(vE_opn, vE_ext);
             case1 = _mm_cmpgt_epi64_rpl(vE_opn, vE_ext);
-            vEM = _mm_blendv_epi8(vEM, vWM, case1);
-            vES = _mm_blendv_epi8(vES, vWS, case1);
-            vEL = _mm_blendv_epi8(vEL, vWL, case1);
+            vEM = _mm_blendv_epi8_rpl(vEM, vWM, case1);
+            vES = _mm_blendv_epi8_rpl(vES, vWS, case1);
+            vEL = _mm_blendv_epi8_rpl(vEL, vWL, case1);
             vEL = _mm_add_epi64(vEL, vOne);
             vs2 = _mm_srli_si128(vs2, 8);
             vs2 = _mm_insert_epi64_rpl(vs2, s2[j], 1);
@@ -373,45 +485,47 @@ parasail_result_t* FNAME(
             vNWH = _mm_add_epi64(vNWH, vMat);
             vWH = _mm_max_epi64_rpl(vNWH, vE);
             vWH = _mm_max_epi64_rpl(vWH, vF);
-            case1 = _mm_cmpeq_epi64(vWH, vNWH);
-            case2 = _mm_cmpeq_epi64(vWH, vF);
-            vWM = _mm_blendv_epi8(
-                    _mm_blendv_epi8(vEM, vFM, case2),
+            case1 = _mm_cmpeq_epi64_rpl(vWH, vNWH);
+            case2 = _mm_cmpeq_epi64_rpl(vWH, vF);
+            vWM = _mm_blendv_epi8_rpl(
+                    _mm_blendv_epi8_rpl(vEM, vFM, case2),
                     _mm_add_epi64(vNWM,
                         _mm_and_si128(
-                            _mm_cmpeq_epi64(vs1,vs2),
+                            _mm_cmpeq_epi64_rpl(vs1,vs2),
                             vOne)),
                     case1);
-            vWS = _mm_blendv_epi8(
-                    _mm_blendv_epi8(vES, vFS, case2),
+            vWS = _mm_blendv_epi8_rpl(
+                    _mm_blendv_epi8_rpl(vES, vFS, case2),
                     _mm_add_epi64(vNWS,
                         _mm_and_si128(
                             _mm_cmpgt_epi64_rpl(vMat,vZero),
                             vOne)),
                     case1);
-            vWL = _mm_blendv_epi8(
-                    _mm_blendv_epi8(vEL, vFL, case2),
+            vWL = _mm_blendv_epi8_rpl(
+                    _mm_blendv_epi8_rpl(vEL, vFL, case2),
                     _mm_add_epi64(vNWL, vOne), case1);
             /* as minor diagonal vector passes across the j=-1 boundary,
              * assign the appropriate boundary conditions */
             {
-                __m128i cond = _mm_cmpeq_epi64(vJ,vNegOne);
-                vWH = _mm_blendv_epi8(vWH, vIBoundary, cond);
+                __m128i cond = _mm_cmpeq_epi64_rpl(vJ,vNegOne);
+                vWH = _mm_blendv_epi8_rpl(vWH, vIBoundary, cond);
                 vWM = _mm_andnot_si128(cond, vWM);
                 vWS = _mm_andnot_si128(cond, vWS);
                 vWL = _mm_andnot_si128(cond, vWL);
-                vE = _mm_blendv_epi8(vE, vNegInf, cond);
+                vE = _mm_blendv_epi8_rpl(vE, vNegInf, cond);
                 vEM = _mm_andnot_si128(cond, vEM);
                 vES = _mm_andnot_si128(cond, vES);
                 vEL = _mm_andnot_si128(cond, vEL);
             }
-            vSaturationCheckMin = _mm_min_epi64_rpl(vSaturationCheckMin, vWH);
-            vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vWH);
-            vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vWM);
-            vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vWS);
-            vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vWL);
-            vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vWL);
-            vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vJ);
+            /* cannot start checking sat until after J clears boundary */
+            if (j > PAD) {
+                vSaturationCheckMin = _mm_min_epi64_rpl(vSaturationCheckMin, vWH);
+                vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vWH);
+                vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vWM);
+                vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vWS);
+                vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vWL);
+                vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vWL);
+            }
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->stats->tables->score_table, vWH, i, s1Len, j, s2Len);
             arr_store_si128(result->stats->tables->matches_table, vWM, i, s1Len, j, s2Len);
@@ -435,18 +549,17 @@ parasail_result_t* FNAME(
             /* as minor diagonal vector passes across table, extract
                last table value at the i,j bound */
             {
-                __m128i cond_valid_I = _mm_cmpeq_epi64(vI, vILimit1);
-                __m128i cond_valid_J = _mm_cmpeq_epi64(vJ, vJLimit1);
+                __m128i cond_valid_I = _mm_cmpeq_epi64_rpl(vI, vILimit1);
+                __m128i cond_valid_J = _mm_cmpeq_epi64_rpl(vJ, vJLimit1);
                 __m128i cond_all = _mm_and_si128(cond_valid_I, cond_valid_J);
-                vMaxH = _mm_blendv_epi8(vMaxH, vWH, cond_all);
-                vMaxM = _mm_blendv_epi8(vMaxM, vWM, cond_all);
-                vMaxS = _mm_blendv_epi8(vMaxS, vWS, cond_all);
-                vMaxL = _mm_blendv_epi8(vMaxL, vWL, cond_all);
+                vMaxH = _mm_blendv_epi8_rpl(vMaxH, vWH, cond_all);
+                vMaxM = _mm_blendv_epi8_rpl(vMaxM, vWM, cond_all);
+                vMaxS = _mm_blendv_epi8_rpl(vMaxS, vWS, cond_all);
+                vMaxL = _mm_blendv_epi8_rpl(vMaxL, vWL, cond_all);
             }
             vJ = _mm_add_epi64(vJ, vOne);
         }
         vI = _mm_add_epi64(vI, vN);
-        vSaturationCheckMax = _mm_max_epi64_rpl(vSaturationCheckMax, vI);
         vIBoundary = _mm_sub_epi64(vIBoundary, vGapN);
     }
 
@@ -484,15 +597,6 @@ parasail_result_t* FNAME(
     result->stats->matches = matches;
     result->stats->similar = similar;
     result->stats->length = length;
-    result->flag |= PARASAIL_FLAG_NW | PARASAIL_FLAG_DIAG
-        | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_2;
-#ifdef PARASAIL_TABLE
-    result->flag |= PARASAIL_FLAG_TABLE;
-#endif
-#ifdef PARASAIL_ROWCOL
-    result->flag |= PARASAIL_FLAG_ROWCOL;
-#endif
 
     parasail_free(_FL_pr);
     parasail_free(_FS_pr);
