@@ -14,7 +14,6 @@
 #include <intrin.h>
 #else
 #include <emmintrin.h>
-#include <smmintrin.h>
 #endif
 
 #include "parasail.h"
@@ -22,6 +21,39 @@
 #include "parasail/internal_sse.h"
 
 
+static inline __m128i _mm_blendv_epi8_rpl(__m128i a, __m128i b, __m128i mask) {
+    a = _mm_andnot_si128(mask, a);
+    a = _mm_or_si128(a, _mm_and_si128(mask, b));
+    return a;
+}
+
+static inline __m128i _mm_insert_epi32_rpl(__m128i a, int32_t i, const int imm) {
+    __m128i_32_t A;
+    A.m = a;
+    A.v[imm] = i;
+    return A.m;
+}
+
+static inline __m128i _mm_max_epi32_rpl(__m128i a, __m128i b) {
+    __m128i mask = _mm_cmpgt_epi32(a, b);
+    a = _mm_and_si128(a, mask);
+    b = _mm_andnot_si128(mask, b);
+    return _mm_or_si128(a, b);
+}
+
+static inline int32_t _mm_extract_epi32_rpl(__m128i a, const int imm) {
+    __m128i_32_t A;
+    A.m = a;
+    return A.v[imm];
+}
+
+static inline __m128i _mm_min_epi32_rpl(__m128i a, __m128i b) {
+    __m128i mask = _mm_cmpgt_epi32(b, a);
+    a = _mm_and_si128(a, mask);
+    b = _mm_andnot_si128(mask, b);
+    return _mm_or_si128(a, b);
+}
+
 
 #ifdef PARASAIL_TABLE
 static inline void arr_store_si128(
@@ -32,10 +64,10 @@ static inline void arr_store_si128(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*(0*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32(vH, 0);
-    array[1LL*(1*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32(vH, 1);
-    array[1LL*(2*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32(vH, 2);
-    array[1LL*(3*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32(vH, 3);
+    array[1LL*(0*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32_rpl(vH, 0);
+    array[1LL*(1*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32_rpl(vH, 1);
+    array[1LL*(2*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32_rpl(vH, 2);
+    array[1LL*(3*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32_rpl(vH, 3);
 }
 #endif
 
@@ -46,23 +78,23 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[0*seglen+t] = (int32_t)_mm_extract_epi32(vH, 0);
-    col[1*seglen+t] = (int32_t)_mm_extract_epi32(vH, 1);
-    col[2*seglen+t] = (int32_t)_mm_extract_epi32(vH, 2);
-    col[3*seglen+t] = (int32_t)_mm_extract_epi32(vH, 3);
+    col[0*seglen+t] = (int32_t)_mm_extract_epi32_rpl(vH, 0);
+    col[1*seglen+t] = (int32_t)_mm_extract_epi32_rpl(vH, 1);
+    col[2*seglen+t] = (int32_t)_mm_extract_epi32_rpl(vH, 2);
+    col[3*seglen+t] = (int32_t)_mm_extract_epi32_rpl(vH, 3);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_nw_stats_table_scan_sse41_128_32
-#define PNAME parasail_nw_stats_table_scan_profile_sse41_128_32
+#define FNAME parasail_nw_stats_table_scan_sse2_128_32
+#define PNAME parasail_nw_stats_table_scan_profile_sse2_128_32
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_nw_stats_rowcol_scan_sse41_128_32
-#define PNAME parasail_nw_stats_rowcol_scan_profile_sse41_128_32
+#define FNAME parasail_nw_stats_rowcol_scan_sse2_128_32
+#define PNAME parasail_nw_stats_rowcol_scan_profile_sse2_128_32
 #else
-#define FNAME parasail_nw_stats_scan_sse41_128_32
-#define PNAME parasail_nw_stats_scan_profile_sse41_128_32
+#define FNAME parasail_nw_stats_scan_sse2_128_32
+#define PNAME parasail_nw_stats_scan_profile_sse2_128_32
 #endif
 #endif
 
@@ -71,9 +103,31 @@ parasail_result_t* FNAME(
         const char * const restrict s2, const int s2Len,
         const int open, const int gap, const parasail_matrix_t *matrix)
 {
-    parasail_profile_t *profile = parasail_profile_create_stats_sse_128_32(s1, s1Len, matrix);
-    parasail_result_t *result = PNAME(profile, s2, s2Len, open, gap);
+    /* declare local variables */
+    parasail_profile_t *profile = NULL;
+    parasail_result_t *result = NULL;
+
+    /* validate inputs */
+    PARASAIL_CHECK_NULL(s2);
+    PARASAIL_CHECK_GT0(s2Len);
+    PARASAIL_CHECK_GE0(open);
+    PARASAIL_CHECK_GE0(gap);
+    PARASAIL_CHECK_NULL(matrix);
+    if (matrix->type == PARASAIL_MATRIX_TYPE_PSSM) {
+        PARASAIL_CHECK_NULL_PSSM_STATS(s1);
+    }
+    else {
+        PARASAIL_CHECK_NULL(s1);
+        PARASAIL_CHECK_GT0(s1Len);
+    }
+
+    /* initialize local variables */
+    profile = parasail_profile_create_stats_sse_128_32(s1, s1Len, matrix);
+    if (!profile) return NULL;
+    result = PNAME(profile, s2, s2Len, open, gap);
+
     parasail_profile_free(profile);
+
     return result;
 }
 
@@ -82,67 +136,157 @@ parasail_result_t* PNAME(
         const char * const restrict s2, const int s2Len,
         const int open, const int gap)
 {
+    /* declare local variables */
     int32_t i = 0;
     int32_t j = 0;
     int32_t k = 0;
-    const int s1Len = profile->s1Len;
-    int32_t end_query = s1Len-1;
-    int32_t end_ref = s2Len-1;
-    const parasail_matrix_t *matrix = profile->matrix;
-    const int32_t segWidth = 4; /* number of values in vector unit */
-    const int32_t segLen = (s1Len + segWidth - 1) / segWidth;
-    const int32_t offset = (s1Len - 1) % segLen;
-    const int32_t position = (segWidth - 1) - (s1Len - 1) / segLen;
-    __m128i* const restrict pvP  = (__m128i*)profile->profile32.score;
-    __m128i* const restrict pvPm = (__m128i*)profile->profile32.matches;
-    __m128i* const restrict pvPs = (__m128i*)profile->profile32.similar;
-    __m128i* const restrict pvE  = parasail_memalign___m128i(16, segLen);
-    __m128i* const restrict pvEM = parasail_memalign___m128i(16, segLen);
-    __m128i* const restrict pvES = parasail_memalign___m128i(16, segLen);
-    __m128i* const restrict pvEL = parasail_memalign___m128i(16, segLen);
-    __m128i* const restrict pvH  = parasail_memalign___m128i(16, segLen);
-    __m128i* const restrict pvHM = parasail_memalign___m128i(16, segLen);
-    __m128i* const restrict pvHS = parasail_memalign___m128i(16, segLen);
-    __m128i* const restrict pvHL = parasail_memalign___m128i(16, segLen);
-    __m128i* const restrict pvHMax  = parasail_memalign___m128i(16, segLen);
-    __m128i* const restrict pvHMMax = parasail_memalign___m128i(16, segLen);
-    __m128i* const restrict pvHSMax = parasail_memalign___m128i(16, segLen);
-    __m128i* const restrict pvHLMax = parasail_memalign___m128i(16, segLen);
-    __m128i* const restrict pvGapper = parasail_memalign___m128i(16, segLen);
-    __m128i* const restrict pvGapperL = parasail_memalign___m128i(16, segLen);
-    int32_t* const restrict boundary = parasail_memalign_int32_t(16, s2Len+1);
-    __m128i vGapO = _mm_set1_epi32(open);
-    __m128i vGapE = _mm_set1_epi32(gap);
-    const int32_t NEG_LIMIT = (-open < matrix->min ?
-        INT32_MIN + open : INT32_MIN - matrix->min) + 1;
-    const int32_t POS_LIMIT = INT32_MAX - matrix->max - 1;
-    __m128i vZero = _mm_setzero_si128();
-    __m128i vOne = _mm_set1_epi32(1);
-    int32_t score = NEG_LIMIT;
+    int32_t s1Len = 0;
+    int32_t end_query = 0;
+    int32_t end_ref = 0;
+    const parasail_matrix_t *matrix = NULL;
+    int32_t segWidth = 0;
+    int32_t segLen = 0;
+    int32_t offset = 0;
+    int32_t position = 0;
+    __m128i* restrict pvP = NULL;
+    __m128i* restrict pvPm = NULL;
+    __m128i* restrict pvPs = NULL;
+    __m128i* restrict pvE = NULL;
+    __m128i* restrict pvEM = NULL;
+    __m128i* restrict pvES = NULL;
+    __m128i* restrict pvEL = NULL;
+    __m128i* restrict pvH = NULL;
+    __m128i* restrict pvHM = NULL;
+    __m128i* restrict pvHS = NULL;
+    __m128i* restrict pvHL = NULL;
+    __m128i* restrict pvHMax = NULL;
+    __m128i* restrict pvHMMax = NULL;
+    __m128i* restrict pvHSMax = NULL;
+    __m128i* restrict pvHLMax = NULL;
+    __m128i* restrict pvGapper = NULL;
+    __m128i* restrict pvGapperL = NULL;
+    int32_t* restrict boundary = NULL;
+    __m128i vGapO;
+    __m128i vGapE;
+    int32_t NEG_LIMIT = 0;
+    int32_t POS_LIMIT = 0;
+    __m128i vZero;
+    __m128i vOne;
+    int32_t score = 0;
     int32_t matches = 0;
     int32_t similar = 0;
     int32_t length = 0;
-    __m128i vNegLimit = _mm_set1_epi32(NEG_LIMIT);
-    __m128i vPosLimit = _mm_set1_epi32(POS_LIMIT);
-    __m128i vSaturationCheckMin = vPosLimit;
-    __m128i vSaturationCheckMax = vNegLimit;
-    __m128i vNegInfFront = vZero;
+    __m128i vNegLimit;
+    __m128i vPosLimit;
+    __m128i vSaturationCheckMin;
+    __m128i vSaturationCheckMax;
+    __m128i vNegInfFront;
     __m128i vSegLenXgap;
-    __m128i vSegLen = _mm_slli_si128(_mm_set1_epi32(segLen), 4);
-#ifdef PARASAIL_TABLE
-    parasail_result_t *result = parasail_result_new_table3(segLen*segWidth, s2Len);
-#else
-#ifdef PARASAIL_ROWCOL
-    parasail_result_t *result = parasail_result_new_rowcol3(segLen*segWidth, s2Len);
-#else
-    parasail_result_t *result = parasail_result_new_stats();
-#endif
-#endif
+    __m128i vSegLen;
+    parasail_result_t *result = NULL;
 
-    vNegInfFront = _mm_insert_epi32(vNegInfFront, NEG_LIMIT, 0);
+    /* validate inputs */
+    PARASAIL_CHECK_NULL(profile);
+    PARASAIL_CHECK_NULL(profile->profile32.score);
+    PARASAIL_CHECK_NULL(profile->matrix);
+    PARASAIL_CHECK_GT0(profile->s1Len);
+    PARASAIL_CHECK_NULL(s2);
+    PARASAIL_CHECK_GT0(s2Len);
+    PARASAIL_CHECK_GE0(open);
+    PARASAIL_CHECK_GE0(gap);
+
+    /* initialize stack variables */
+    i = 0;
+    j = 0;
+    k = 0;
+    s1Len = profile->s1Len;
+    end_query = s1Len-1;
+    end_ref = s2Len-1;
+    matrix = profile->matrix;
+    segWidth = 4; /* number of values in vector unit */
+    segLen = (s1Len + segWidth - 1) / segWidth;
+    offset = (s1Len - 1) % segLen;
+    position = (segWidth - 1) - (s1Len - 1) / segLen;
+    pvP  = (__m128i*)profile->profile32.score;
+    pvPm = (__m128i*)profile->profile32.matches;
+    pvPs = (__m128i*)profile->profile32.similar;
+    vGapO = _mm_set1_epi32(open);
+    vGapE = _mm_set1_epi32(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
+    POS_LIMIT = INT32_MAX - matrix->max - 1;
+    vZero = _mm_setzero_si128();
+    vOne = _mm_set1_epi32(1);
+    score = NEG_LIMIT;
+    matches = 0;
+    similar = 0;
+    length = 0;
+    vNegLimit = _mm_set1_epi32(NEG_LIMIT);
+    vPosLimit = _mm_set1_epi32(POS_LIMIT);
+    vSaturationCheckMin = vPosLimit;
+    vSaturationCheckMax = vNegLimit;
+    vNegInfFront = vZero;
+    vSegLen = _mm_slli_si128(_mm_set1_epi32(segLen), 4);
+    vNegInfFront = _mm_insert_epi32_rpl(vNegInfFront, NEG_LIMIT, 0);
     vSegLenXgap = _mm_add_epi32(vNegInfFront,
             _mm_slli_si128(_mm_set1_epi32(-segLen*gap), 4));
 
+    /* initialize result */
+#ifdef PARASAIL_TABLE
+    result = parasail_result_new_table3(segLen*segWidth, s2Len);
+#else
+#ifdef PARASAIL_ROWCOL
+    result = parasail_result_new_rowcol3(segLen*segWidth, s2Len);
+#else
+    result = parasail_result_new_stats();
+#endif
+#endif
+    if (!result) return NULL;
+
+    /* set known flags */
+    result->flag |= PARASAIL_FLAG_NW | PARASAIL_FLAG_SCAN
+        | PARASAIL_FLAG_STATS
+        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_4;
+#ifdef PARASAIL_TABLE
+    result->flag |= PARASAIL_FLAG_TABLE;
+#endif
+#ifdef PARASAIL_ROWCOL
+    result->flag |= PARASAIL_FLAG_ROWCOL;
+#endif
+
+    /* initialize heap variables */
+    pvE  = parasail_memalign___m128i(16, segLen);
+    pvEM = parasail_memalign___m128i(16, segLen);
+    pvES = parasail_memalign___m128i(16, segLen);
+    pvEL = parasail_memalign___m128i(16, segLen);
+    pvH  = parasail_memalign___m128i(16, segLen);
+    pvHM = parasail_memalign___m128i(16, segLen);
+    pvHS = parasail_memalign___m128i(16, segLen);
+    pvHL = parasail_memalign___m128i(16, segLen);
+    pvHMax  = parasail_memalign___m128i(16, segLen);
+    pvHMMax = parasail_memalign___m128i(16, segLen);
+    pvHSMax = parasail_memalign___m128i(16, segLen);
+    pvHLMax = parasail_memalign___m128i(16, segLen);
+    pvGapper = parasail_memalign___m128i(16, segLen);
+    pvGapperL = parasail_memalign___m128i(16, segLen);
+    boundary = parasail_memalign_int32_t(16, s2Len+1);
+
+    /* validate heap variables */
+    if (!pvE) return NULL;
+    if (!pvEM) return NULL;
+    if (!pvES) return NULL;
+    if (!pvEL) return NULL;
+    if (!pvH) return NULL;
+    if (!pvHM) return NULL;
+    if (!pvHS) return NULL;
+    if (!pvHL) return NULL;
+    if (!pvHMax) return NULL;
+    if (!pvHMMax) return NULL;
+    if (!pvHSMax) return NULL;
+    if (!pvHLMax) return NULL;
+    if (!pvGapper) return NULL;
+    if (!pvGapperL) return NULL;
+    if (!boundary) return NULL;
+
     parasail_memset___m128i(pvHM, vZero, segLen);
     parasail_memset___m128i(pvHS, vZero, segLen);
     parasail_memset___m128i(pvHL, vZero, segLen);
@@ -233,7 +377,7 @@ parasail_result_t* PNAME(
         vHpM = _mm_slli_si128(vHpM, 4);
         vHpS = _mm_slli_si128(vHpS, 4);
         vHpL = _mm_slli_si128(vHpL, 4);
-        vHp = _mm_insert_epi32(vHp, boundary[j], 0);
+        vHp = _mm_insert_epi32_rpl(vHp, boundary[j], 0);
         pvW = pvP + matrix->mapper[(unsigned char)s2[j]]*segLen;
         pvWM= pvPm+ matrix->mapper[(unsigned char)s2[j]]*segLen;
         pvWS= pvPs+ matrix->mapper[(unsigned char)s2[j]]*segLen;
@@ -259,19 +403,19 @@ parasail_result_t* PNAME(
             vE_opn = _mm_sub_epi32(vH, vGapO);
             vE_ext = _mm_sub_epi32(vE, vGapE);
             case1 = _mm_cmpgt_epi32(vE_opn, vE_ext);
-            vE = _mm_max_epi32(vE_opn, vE_ext);
-            vEM = _mm_blendv_epi8(vEM, vHM, case1);
-            vES = _mm_blendv_epi8(vES, vHS, case1);
-            vEL = _mm_blendv_epi8(vEL, vHL, case1);
+            vE = _mm_max_epi32_rpl(vE_opn, vE_ext);
+            vEM = _mm_blendv_epi8_rpl(vEM, vHM, case1);
+            vES = _mm_blendv_epi8_rpl(vES, vHS, case1);
+            vEL = _mm_blendv_epi8_rpl(vEL, vHL, case1);
             vEL = _mm_add_epi32(vEL, vOne);
             vGapper = _mm_add_epi32(vHt, vGapper);
             case1 = _mm_or_si128(
                     _mm_cmpgt_epi32(vF, vGapper),
                     _mm_cmpeq_epi32(vF, vGapper));
-            vF = _mm_max_epi32(vF, vGapper);
-            vFM = _mm_blendv_epi8(vHtM, vFM, case1);
-            vFS = _mm_blendv_epi8(vHtS, vFS, case1);
-            vFL = _mm_blendv_epi8(
+            vF = _mm_max_epi32_rpl(vF, vGapper);
+            vFM = _mm_blendv_epi8_rpl(vHtM, vFM, case1);
+            vFS = _mm_blendv_epi8_rpl(vHtS, vFS, case1);
+            vFL = _mm_blendv_epi8_rpl(
                     _mm_add_epi32(vHtL, vGapperL),
                     vFL, case1);
             vHp = _mm_add_epi32(vHp, vW);
@@ -279,10 +423,10 @@ parasail_result_t* PNAME(
             vHpS = _mm_add_epi32(vHpS, vWS);
             vHpL = _mm_add_epi32(vHpL, vOne);
             case1 = _mm_cmpgt_epi32(vE, vHp);
-            vHt = _mm_max_epi32(vE, vHp);
-            vHtM = _mm_blendv_epi8(vHpM, vEM, case1);
-            vHtS = _mm_blendv_epi8(vHpS, vES, case1);
-            vHtL = _mm_blendv_epi8(vHpL, vEL, case1);
+            vHt = _mm_max_epi32_rpl(vE, vHp);
+            vHtM = _mm_blendv_epi8_rpl(vHpM, vEM, case1);
+            vHtS = _mm_blendv_epi8_rpl(vHpS, vES, case1);
+            vHtL = _mm_blendv_epi8_rpl(vHpL, vEL, case1);
             _mm_store_si128(pvE+i, vE);
             _mm_store_si128(pvEM+i, vEM);
             _mm_store_si128(pvES+i, vES);
@@ -302,17 +446,17 @@ parasail_result_t* PNAME(
         vHtM = _mm_slli_si128(vHtM, 4);
         vHtS = _mm_slli_si128(vHtS, 4);
         vHtL = _mm_slli_si128(vHtL, 4);
-        vHt = _mm_insert_epi32(vHt, boundary[j+1], 0);
+        vHt = _mm_insert_epi32_rpl(vHt, boundary[j+1], 0);
         vGapper = _mm_load_si128(pvGapper);
         vGapperL = _mm_load_si128(pvGapperL);
         vGapper = _mm_add_epi32(vHt, vGapper);
         case1 = _mm_or_si128(
                 _mm_cmpgt_epi32(vGapper, vF),
                 _mm_cmpeq_epi32(vGapper, vF));
-        vF = _mm_max_epi32(vF, vGapper);
-        vFM = _mm_blendv_epi8(vFM, vHtM, case1);
-        vFS = _mm_blendv_epi8(vFS, vHtS, case1);
-        vFL = _mm_blendv_epi8(
+        vF = _mm_max_epi32_rpl(vF, vGapper);
+        vFM = _mm_blendv_epi8_rpl(vFM, vHtM, case1);
+        vFS = _mm_blendv_epi8_rpl(vFS, vHtS, case1);
+        vFL = _mm_blendv_epi8_rpl(
                 vFL,
                 _mm_add_epi32(vHtL, vGapperL),
                 case1);
@@ -325,10 +469,10 @@ parasail_result_t* PNAME(
             case1 = _mm_or_si128(
                     _mm_cmpgt_epi32(vFt, vF),
                     _mm_cmpeq_epi32(vFt, vF));
-            vF = _mm_max_epi32(vF, vFt);
-            vFM = _mm_blendv_epi8(vFM, vFtM, case1);
-            vFS = _mm_blendv_epi8(vFS, vFtS, case1);
-            vFL = _mm_blendv_epi8(
+            vF = _mm_max_epi32_rpl(vF, vFt);
+            vFM = _mm_blendv_epi8_rpl(vFM, vFtM, case1);
+            vFS = _mm_blendv_epi8_rpl(vFS, vFtS, case1);
+            vFL = _mm_blendv_epi8_rpl(
                     vFL,
                     _mm_add_epi32(vFtL, vSegLen),
                     case1);
@@ -341,10 +485,10 @@ parasail_result_t* PNAME(
         vFL = _mm_slli_si128(vFL, 4);
         vF = _mm_add_epi32(vF, vNegInfFront);
         case1 = _mm_cmpgt_epi32(vF, vHt);
-        vH = _mm_max_epi32(vF, vHt);
-        vHM = _mm_blendv_epi8(vHtM, vFM, case1);
-        vHS = _mm_blendv_epi8(vHtS, vFS, case1);
-        vHL = _mm_blendv_epi8(vHtL, vFL, case1);
+        vH = _mm_max_epi32_rpl(vF, vHt);
+        vHM = _mm_blendv_epi8_rpl(vHtM, vFM, case1);
+        vHS = _mm_blendv_epi8_rpl(vHtS, vFS, case1);
+        vHL = _mm_blendv_epi8_rpl(vHtL, vFL, case1);
         for (i=0; i<segLen; ++i) {
             vHp = _mm_load_si128(pvH+i);
             vHpM = _mm_load_si128(pvHM+i);
@@ -356,34 +500,34 @@ parasail_result_t* PNAME(
             vEL = _mm_load_si128(pvEL+i);
             vF_opn = _mm_sub_epi32(vH, vGapO);
             vF_ext = _mm_sub_epi32(vF, vGapE);
-            vF = _mm_max_epi32(vF_opn, vF_ext);
+            vF = _mm_max_epi32_rpl(vF_opn, vF_ext);
             case1 = _mm_cmpgt_epi32(vF_opn, vF_ext);
-            vFM = _mm_blendv_epi8(vFM, vHM, case1);
-            vFS = _mm_blendv_epi8(vFS, vHS, case1);
-            vFL = _mm_blendv_epi8(vFL, vHL, case1);
+            vFM = _mm_blendv_epi8_rpl(vFM, vHM, case1);
+            vFS = _mm_blendv_epi8_rpl(vFS, vHS, case1);
+            vFL = _mm_blendv_epi8_rpl(vFL, vHL, case1);
             vFL = _mm_add_epi32(vFL, vOne);
-            vH = _mm_max_epi32(vHp, vE);
-            vH = _mm_max_epi32(vH, vF);
+            vH = _mm_max_epi32_rpl(vHp, vE);
+            vH = _mm_max_epi32_rpl(vH, vF);
             case1 = _mm_cmpeq_epi32(vH, vHp);
             case2 = _mm_cmpeq_epi32(vH, vF);
-            vHM = _mm_blendv_epi8(
-                    _mm_blendv_epi8(vEM, vFM, case2),
+            vHM = _mm_blendv_epi8_rpl(
+                    _mm_blendv_epi8_rpl(vEM, vFM, case2),
                     vHpM, case1);
-            vHS = _mm_blendv_epi8(
-                    _mm_blendv_epi8(vES, vFS, case2),
+            vHS = _mm_blendv_epi8_rpl(
+                    _mm_blendv_epi8_rpl(vES, vFS, case2),
                     vHpS, case1);
-            vHL = _mm_blendv_epi8(
-                    _mm_blendv_epi8(vEL, vFL, case2),
+            vHL = _mm_blendv_epi8_rpl(
+                    _mm_blendv_epi8_rpl(vEL, vFL, case2),
                     vHpL, case1);
             _mm_store_si128(pvH+i, vH);
             _mm_store_si128(pvHM+i, vHM);
             _mm_store_si128(pvHS+i, vHS);
             _mm_store_si128(pvHL+i, vHL);
-            vSaturationCheckMin = _mm_min_epi32(vSaturationCheckMin, vH);
-            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vH);
-            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vHM);
-            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vHS);
-            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vHL);
+            vSaturationCheckMin = _mm_min_epi32_rpl(vSaturationCheckMin, vH);
+            vSaturationCheckMax = _mm_max_epi32_rpl(vSaturationCheckMax, vH);
+            vSaturationCheckMax = _mm_max_epi32_rpl(vSaturationCheckMax, vHM);
+            vSaturationCheckMax = _mm_max_epi32_rpl(vSaturationCheckMax, vHS);
+            vSaturationCheckMax = _mm_max_epi32_rpl(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
             arr_store_si128(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
@@ -405,10 +549,10 @@ parasail_result_t* PNAME(
                 vHS = _mm_slli_si128(vHS, 4);
                 vHL = _mm_slli_si128(vHL, 4);
             }
-            result->stats->rowcols->score_row[j] = (int32_t) _mm_extract_epi32 (vH, 3);
-            result->stats->rowcols->matches_row[j] = (int32_t) _mm_extract_epi32 (vHM, 3);
-            result->stats->rowcols->similar_row[j] = (int32_t) _mm_extract_epi32 (vHS, 3);
-            result->stats->rowcols->length_row[j] = (int32_t) _mm_extract_epi32 (vHL, 3);
+            result->stats->rowcols->score_row[j] = (int32_t) _mm_extract_epi32_rpl (vH, 3);
+            result->stats->rowcols->matches_row[j] = (int32_t) _mm_extract_epi32_rpl (vHM, 3);
+            result->stats->rowcols->similar_row[j] = (int32_t) _mm_extract_epi32_rpl (vHS, 3);
+            result->stats->rowcols->length_row[j] = (int32_t) _mm_extract_epi32_rpl (vHL, 3);
         }
 #endif
     }
@@ -438,10 +582,10 @@ parasail_result_t* PNAME(
             vHS = _mm_slli_si128(vHS, 4);
             vHL = _mm_slli_si128(vHL, 4);
         }
-        score = (int32_t) _mm_extract_epi32 (vH, 3);
-        matches = (int32_t) _mm_extract_epi32 (vHM, 3);
-        similar = (int32_t) _mm_extract_epi32 (vHS, 3);
-        length = (int32_t) _mm_extract_epi32 (vHL, 3);
+        score = (int32_t) _mm_extract_epi32_rpl (vH, 3);
+        matches = (int32_t) _mm_extract_epi32_rpl (vHM, 3);
+        similar = (int32_t) _mm_extract_epi32_rpl (vHS, 3);
+        length = (int32_t) _mm_extract_epi32_rpl (vHL, 3);
     }
 
     if (_mm_movemask_epi8(_mm_or_si128(
@@ -462,15 +606,6 @@ parasail_result_t* PNAME(
     result->stats->matches = matches;
     result->stats->similar = similar;
     result->stats->length = length;
-    result->flag |= PARASAIL_FLAG_NW | PARASAIL_FLAG_SCAN
-        | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_4;
-#ifdef PARASAIL_TABLE
-    result->flag |= PARASAIL_FLAG_TABLE;
-#endif
-#ifdef PARASAIL_ROWCOL
-    result->flag |= PARASAIL_FLAG_ROWCOL;
-#endif
 
     parasail_free(boundary);
     parasail_free(pvGapperL);
