@@ -14,7 +14,6 @@
 #include <intrin.h>
 #else
 #include <emmintrin.h>
-#include <smmintrin.h>
 #endif
 
 #include "parasail.h"
@@ -22,6 +21,12 @@
 #include "parasail/internal_sse.h"
 
 
+static inline __m128i _mm_blendv_epi8_rpl(__m128i a, __m128i b, __m128i mask) {
+    a = _mm_andnot_si128(mask, a);
+    a = _mm_or_si128(a, _mm_and_si128(mask, b));
+    return a;
+}
+
 
 static inline void arr_store(
         __m128i *array,
@@ -42,17 +47,36 @@ static inline __m128i arr_load(
     return _mm_load_si128(array + (1LL*d*seglen+t));
 }
 
-#define FNAME parasail_nw_trace_scan_sse41_128_16
-#define PNAME parasail_nw_trace_scan_profile_sse41_128_16
+#define FNAME parasail_nw_trace_scan_sse2_128_16
+#define PNAME parasail_nw_trace_scan_profile_sse2_128_16
 
 parasail_result_t* FNAME(
         const char * const restrict s1, const int s1Len,
         const char * const restrict s2, const int s2Len,
         const int open, const int gap, const parasail_matrix_t *matrix)
 {
-    parasail_profile_t *profile = parasail_profile_create_sse_128_16(s1, s1Len, matrix);
-    parasail_result_t *result = PNAME(profile, s2, s2Len, open, gap);
+    /* declare local variables */
+    parasail_profile_t *profile = NULL;
+    parasail_result_t *result = NULL;
+
+    /* validate inputs */
+    PARASAIL_CHECK_NULL(s2);
+    PARASAIL_CHECK_GT0(s2Len);
+    PARASAIL_CHECK_GE0(open);
+    PARASAIL_CHECK_GE0(gap);
+    PARASAIL_CHECK_NULL(matrix);
+    if (matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) {
+        PARASAIL_CHECK_NULL(s1);
+        PARASAIL_CHECK_GT0(s1Len);
+    }
+
+    /* initialize local variables */
+    profile = parasail_profile_create_sse_128_16(s1, s1Len, matrix);
+    if (!profile) return NULL;
+    result = PNAME(profile, s2, s2Len, open, gap);
+
     parasail_profile_free(profile);
+
     return result;
 }
 
@@ -61,49 +85,113 @@ parasail_result_t* PNAME(
         const char * const restrict s2, const int s2Len,
         const int open, const int gap)
 {
+    /* declare local variables */
     int32_t i = 0;
     int32_t j = 0;
     int32_t k = 0;
-    const int s1Len = profile->s1Len;
-    int32_t end_query = s1Len-1;
-    int32_t end_ref = s2Len-1;
-    const parasail_matrix_t *matrix = profile->matrix;
-    const int32_t segWidth = 8; /* number of values in vector unit */
-    const int32_t segLen = (s1Len + segWidth - 1) / segWidth;
-    const int32_t offset = (s1Len - 1) % segLen;
-    const int32_t position = (segWidth - 1) - (s1Len - 1) / segLen;
-    __m128i* const restrict pvP  = (__m128i*)profile->profile16.score;
-    __m128i* const restrict pvE  = parasail_memalign___m128i(16, segLen);
-    int16_t* const restrict boundary = parasail_memalign_int16_t(16, s2Len+1);
-    __m128i* const restrict pvHt = parasail_memalign___m128i(16, segLen);
-    __m128i* const restrict pvH  = parasail_memalign___m128i(16, segLen);
-    __m128i* const restrict pvGapper = parasail_memalign___m128i(16, segLen);
-    __m128i vGapO = _mm_set1_epi16(open);
-    __m128i vGapE = _mm_set1_epi16(gap);
-    const int16_t NEG_LIMIT = (-open < matrix->min ?
-        INT16_MIN + open : INT16_MIN - matrix->min) + 1;
-    const int16_t POS_LIMIT = INT16_MAX - matrix->max - 1;
-    __m128i vZero = _mm_setzero_si128();
-    int16_t score = NEG_LIMIT;
-    __m128i vNegLimit = _mm_set1_epi16(NEG_LIMIT);
-    __m128i vPosLimit = _mm_set1_epi16(POS_LIMIT);
-    __m128i vSaturationCheckMin = vPosLimit;
-    __m128i vSaturationCheckMax = vNegLimit;
-    __m128i vNegInfFront = vZero;
+    int32_t s1Len = 0;
+    int32_t end_query = 0;
+    int32_t end_ref = 0;
+    const parasail_matrix_t *matrix = NULL;
+    int32_t segWidth = 0;
+    int32_t segLen = 0;
+    int32_t offset = 0;
+    int32_t position = 0;
+    __m128i* restrict pvP = NULL;
+    __m128i* restrict pvE = NULL;
+    int16_t* restrict boundary = NULL;
+    __m128i* restrict pvHt = NULL;
+    __m128i* restrict pvH = NULL;
+    __m128i* restrict pvGapper = NULL;
+    __m128i vGapO;
+    __m128i vGapE;
+    int16_t NEG_LIMIT = 0;
+    int16_t POS_LIMIT = 0;
+    __m128i vZero;
+    int16_t score = 0;
+    __m128i vNegLimit;
+    __m128i vPosLimit;
+    __m128i vSaturationCheckMin;
+    __m128i vSaturationCheckMax;
+    __m128i vNegInfFront;
     __m128i vSegLenXgap;
-    parasail_result_t *result = parasail_result_new_trace(segLen, s2Len, 16, sizeof(__m128i));
-    __m128i vTIns  = _mm_set1_epi16(PARASAIL_INS);
-    __m128i vTDel  = _mm_set1_epi16(PARASAIL_DEL);
-    __m128i vTDiag = _mm_set1_epi16(PARASAIL_DIAG);
-    __m128i vTDiagE = _mm_set1_epi16(PARASAIL_DIAG_E);
-    __m128i vTInsE = _mm_set1_epi16(PARASAIL_INS_E);
-    __m128i vTDiagF = _mm_set1_epi16(PARASAIL_DIAG_F);
-    __m128i vTDelF = _mm_set1_epi16(PARASAIL_DEL_F);
+    parasail_result_t *result = NULL;
+    __m128i vTIns;
+    __m128i vTDel;
+    __m128i vTDiag;
+    __m128i vTDiagE;
+    __m128i vTInsE;
+    __m128i vTDiagF;
+    __m128i vTDelF;
 
+    /* validate inputs */
+    PARASAIL_CHECK_NULL(profile);
+    PARASAIL_CHECK_NULL(profile->profile16.score);
+    PARASAIL_CHECK_NULL(profile->matrix);
+    PARASAIL_CHECK_GT0(profile->s1Len);
+    PARASAIL_CHECK_NULL(s2);
+    PARASAIL_CHECK_GT0(s2Len);
+    PARASAIL_CHECK_GE0(open);
+    PARASAIL_CHECK_GE0(gap);
+
+    /* initialize stack variables */
+    i = 0;
+    j = 0;
+    k = 0;
+    s1Len = profile->s1Len;
+    end_query = s1Len-1;
+    end_ref = s2Len-1;
+    matrix = profile->matrix;
+    segWidth = 8; /* number of values in vector unit */
+    segLen = (s1Len + segWidth - 1) / segWidth;
+    offset = (s1Len - 1) % segLen;
+    position = (segWidth - 1) - (s1Len - 1) / segLen;
+    pvP = (__m128i*)profile->profile16.score;
+    vGapO = _mm_set1_epi16(open);
+    vGapE = _mm_set1_epi16(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT16_MIN + open : INT16_MIN - matrix->min) + 1;
+    POS_LIMIT = INT16_MAX - matrix->max - 1;
+    vZero = _mm_setzero_si128();
+    score = NEG_LIMIT;
+    vNegLimit = _mm_set1_epi16(NEG_LIMIT);
+    vPosLimit = _mm_set1_epi16(POS_LIMIT);
+    vSaturationCheckMin = vPosLimit;
+    vSaturationCheckMax = vNegLimit;
+    vNegInfFront = vZero;
+    vTIns  = _mm_set1_epi16(PARASAIL_INS);
+    vTDel  = _mm_set1_epi16(PARASAIL_DEL);
+    vTDiag = _mm_set1_epi16(PARASAIL_DIAG);
+    vTDiagE = _mm_set1_epi16(PARASAIL_DIAG_E);
+    vTInsE = _mm_set1_epi16(PARASAIL_INS_E);
+    vTDiagF = _mm_set1_epi16(PARASAIL_DIAG_F);
+    vTDelF = _mm_set1_epi16(PARASAIL_DEL_F);
     vNegInfFront = _mm_insert_epi16(vNegInfFront, NEG_LIMIT, 0);
-    vSegLenXgap = _mm_add_epi16(vNegInfFront,
+    vSegLenXgap = _mm_adds_epi16(vNegInfFront,
             _mm_slli_si128(_mm_set1_epi16(-segLen*gap), 2));
 
+    /* initialize result */
+    result = parasail_result_new_trace(segLen, s2Len, 16, sizeof(__m128i));
+    if (!result) return NULL;
+
+    /* set known flags */
+    result->flag |= PARASAIL_FLAG_NW | PARASAIL_FLAG_SCAN
+        | PARASAIL_FLAG_TRACE
+        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_8;
+
+    /* initialize heap variables */
+    pvE = parasail_memalign___m128i(16, segLen);
+    boundary = parasail_memalign_int16_t(16, s2Len+1);
+    pvHt= parasail_memalign___m128i(16, segLen);
+    pvH = parasail_memalign___m128i(16, segLen);
+    pvGapper = parasail_memalign___m128i(16, segLen);
+
+    /* validate heap variables */
+    if (!pvE) return NULL;
+    if (!boundary) return NULL;
+    if (!pvHt) return NULL;
+    if (!pvH) return NULL;
+    if (!pvGapper) return NULL;
+
     /* initialize H and E */
     {
         int32_t index = 0;
@@ -133,10 +221,10 @@ parasail_result_t* PNAME(
     }
 
     {
-        __m128i vGapper = _mm_sub_epi16(vZero,vGapO);
+        __m128i vGapper = _mm_subs_epi16(vZero,vGapO);
         for (i=segLen-1; i>=0; --i) {
             _mm_store_si128(pvGapper+i, vGapper);
-            vGapper = _mm_sub_epi16(vGapper, vGapE);
+            vGapper = _mm_subs_epi16(vGapper, vGapE);
         }
     }
 
@@ -167,23 +255,23 @@ parasail_result_t* PNAME(
         vHp = _mm_slli_si128(vHp, 2);
         vHp = _mm_insert_epi16(vHp, boundary[j], 0);
         pvW = pvP + matrix->mapper[(unsigned char)s2[j]]*segLen;
-        vHt = _mm_sub_epi16(vNegLimit, pvGapper[0]);
+        vHt = _mm_subs_epi16(vNegLimit, pvGapper[0]);
         vF = vNegLimit;
         for (i=0; i<segLen; ++i) {
             vH = _mm_load_si128(pvH+i);
             vE = _mm_load_si128(pvE+i);
             vW = _mm_load_si128(pvW+i);
             vGapper = _mm_load_si128(pvGapper+i);
-            vE_opn = _mm_sub_epi16(vH, vGapO);
-            vE_ext = _mm_sub_epi16(vE, vGapE);
+            vE_opn = _mm_subs_epi16(vH, vGapO);
+            vE_ext = _mm_subs_epi16(vE, vGapE);
             case1 = _mm_cmpgt_epi16(vE_opn, vE_ext);
-            vET = _mm_blendv_epi8(vTInsE, vTDiagE, case1);
+            vET = _mm_blendv_epi8_rpl(vTInsE, vTDiagE, case1);
             arr_store(result->trace->trace_table, vET, i, segLen, j);
             vE = _mm_max_epi16(vE_opn, vE_ext);
             vSaturationCheckMin = _mm_min_epi16(vSaturationCheckMin, vE);
-            vGapper = _mm_add_epi16(vHt, vGapper);
+            vGapper = _mm_adds_epi16(vHt, vGapper);
             vF = _mm_max_epi16(vF, vGapper);
-            vHp = _mm_add_epi16(vHp, vW);
+            vHp = _mm_adds_epi16(vHp, vW);
             vHt = _mm_max_epi16(vE, vHp);
             _mm_store_si128(pvE+i, vE);
             _mm_store_si128(pvHt+i, vHt);
@@ -195,32 +283,32 @@ parasail_result_t* PNAME(
         vHt = _mm_slli_si128(vHt, 2);
         vHt = _mm_insert_epi16(vHt, boundary[j+1], 0);
         vGapper = _mm_load_si128(pvGapper);
-        vGapper = _mm_add_epi16(vHt, vGapper);
+        vGapper = _mm_adds_epi16(vHt, vGapper);
         vF = _mm_max_epi16(vF, vGapper);
         for (i=0; i<segWidth-2; ++i) {
             __m128i vFt = _mm_slli_si128(vF, 2);
-            vFt = _mm_add_epi16(vFt, vSegLenXgap);
+            vFt = _mm_adds_epi16(vFt, vSegLenXgap);
             vF = _mm_max_epi16(vF, vFt);
         }
 
         /* calculate final H */
         vF = _mm_slli_si128(vF, 2);
-        vF = _mm_add_epi16(vF, vNegInfFront);
+        vF = _mm_adds_epi16(vF, vNegInfFront);
         vH = _mm_max_epi16(vF, vHt);
         for (i=0; i<segLen; ++i) {
             vET = arr_load(result->trace->trace_table, i, segLen, j);
             vHp = _mm_load_si128(pvH+i);
             vHt = _mm_load_si128(pvHt+i);
-            vF_opn = _mm_sub_epi16(vH, vGapO);
-            vF_ext = _mm_sub_epi16(vF, vGapE);
+            vF_opn = _mm_subs_epi16(vH, vGapO);
+            vF_ext = _mm_subs_epi16(vF, vGapE);
             vF = _mm_max_epi16(vF_opn, vF_ext);
             case1 = _mm_cmpgt_epi16(vF_opn, vF_ext);
-            vFT = _mm_blendv_epi8(vTDelF, vTDiagF, case1);
+            vFT = _mm_blendv_epi8_rpl(vTDelF, vTDiagF, case1);
             vH = _mm_max_epi16(vHt, vF);
             case1 = _mm_cmpeq_epi16(vH, vHp);
             case2 = _mm_cmpeq_epi16(vH, vF);
-            vT = _mm_blendv_epi8(
-                    _mm_blendv_epi8(vTIns, vTDel, case2),
+            vT = _mm_blendv_epi8_rpl(
+                    _mm_blendv_epi8_rpl(vTIns, vTDel, case2),
                     vTDiag, case1);
             vT = _mm_or_si128(vT, vET);
             vT = _mm_or_si128(vT, vFT);
@@ -253,9 +341,6 @@ parasail_result_t* PNAME(
     result->score = score;
     result->end_query = end_query;
     result->end_ref = end_ref;
-    result->flag |= PARASAIL_FLAG_NW | PARASAIL_FLAG_SCAN
-        | PARASAIL_FLAG_TRACE
-        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_8;
 
     parasail_free(pvGapper);
     parasail_free(pvH);
