@@ -19,7 +19,6 @@
 #define SG_SUFFIX _diag_avx2_256_16
 #include "sg_helper.h"
 
-#define NEG_INF (INT16_MIN/(int16_t)(2))
 
 #define _mm256_cmplt_epi16_rpl(a,b) _mm256_cmpgt_epi16(b,a)
 
@@ -227,88 +226,146 @@ static inline void arr_store_rowcol(
 #endif
 
 parasail_result_t* FNAME(
-        const char * const restrict _s1, const int s1Len,
+        const char * const restrict _s1, const int _s1Len,
         const char * const restrict _s2, const int s2Len,
         const int open, const int gap, const parasail_matrix_t *matrix,
         int s1_beg, int s1_end, int s2_beg, int s2_end)
 {
-    const int32_t N = 16; /* number of values in vector */
-    const int32_t PAD = N-1;
-    const int32_t PAD2 = PAD*2;
-    const int32_t s1Len_PAD = s1Len+PAD;
-    const int32_t s2Len_PAD = s2Len+PAD;
-    int16_t * const restrict s1      = parasail_memalign_int16_t(32, s1Len+PAD);
-    int16_t * const restrict s2B     = parasail_memalign_int16_t(32, s2Len+PAD2);
-    int16_t * const restrict _H_pr = parasail_memalign_int16_t(32, s2Len+PAD2);
-    int16_t * const restrict _HM_pr = parasail_memalign_int16_t(32, s2Len+PAD2);
-    int16_t * const restrict _HS_pr = parasail_memalign_int16_t(32, s2Len+PAD2);
-    int16_t * const restrict _HL_pr = parasail_memalign_int16_t(32, s2Len+PAD2);
-    int16_t * const restrict _F_pr = parasail_memalign_int16_t(32, s2Len+PAD2);
-    int16_t * const restrict _FM_pr = parasail_memalign_int16_t(32, s2Len+PAD2);
-    int16_t * const restrict _FS_pr = parasail_memalign_int16_t(32, s2Len+PAD2);
-    int16_t * const restrict _FL_pr = parasail_memalign_int16_t(32, s2Len+PAD2);
-    int16_t * const restrict s2 = s2B+PAD; /* will allow later for negative indices */
-    int16_t * const restrict H_pr = _H_pr+PAD;
-    int16_t * const restrict HM_pr = _HM_pr+PAD;
-    int16_t * const restrict HS_pr = _HS_pr+PAD;
-    int16_t * const restrict HL_pr = _HL_pr+PAD;
-    int16_t * const restrict F_pr = _F_pr+PAD;
-    int16_t * const restrict FM_pr = _FM_pr+PAD;
-    int16_t * const restrict FS_pr = _FS_pr+PAD;
-    int16_t * const restrict FL_pr = _FL_pr+PAD;
-#ifdef PARASAIL_TABLE
-    parasail_result_t *result = parasail_result_new_table3(s1Len, s2Len);
-#else
-#ifdef PARASAIL_ROWCOL
-    parasail_result_t *result = parasail_result_new_rowcol3(s1Len, s2Len);
-#else
-    parasail_result_t *result = parasail_result_new_stats();
-#endif
-#endif
+    /* declare local variables */
+    int32_t N = 0;
+    int32_t PAD = 0;
+    int32_t PAD2 = 0;
+    int32_t s1Len = 0;
+    int32_t s1Len_PAD = 0;
+    int32_t s2Len_PAD = 0;
+    int16_t * restrict s1 = NULL;
+    int16_t * restrict s2B = NULL;
+    int16_t * restrict _H_pr = NULL;
+    int16_t * restrict _HM_pr = NULL;
+    int16_t * restrict _HS_pr = NULL;
+    int16_t * restrict _HL_pr = NULL;
+    int16_t * restrict _F_pr = NULL;
+    int16_t * restrict _FM_pr = NULL;
+    int16_t * restrict _FS_pr = NULL;
+    int16_t * restrict _FL_pr = NULL;
+    int16_t * restrict s2 = NULL;
+    int16_t * restrict H_pr = NULL;
+    int16_t * restrict HM_pr = NULL;
+    int16_t * restrict HS_pr = NULL;
+    int16_t * restrict HL_pr = NULL;
+    int16_t * restrict F_pr = NULL;
+    int16_t * restrict FM_pr = NULL;
+    int16_t * restrict FS_pr = NULL;
+    int16_t * restrict FL_pr = NULL;
+    parasail_result_t *result = NULL;
     int32_t i = 0;
     int32_t j = 0;
-    int32_t end_query = s1Len-1;
-    int32_t end_ref = s2Len-1;
-    const int16_t NEG_LIMIT = (-open < matrix->min ?
-        INT16_MIN + open : INT16_MIN - matrix->min) + 1;
-    const int16_t POS_LIMIT = INT16_MAX - matrix->max - 1;
-    int16_t score = NEG_LIMIT;
-    int16_t matches = NEG_LIMIT;
-    int16_t similar = NEG_LIMIT;
-    int16_t length = NEG_LIMIT;
-    __m256i vNegLimit = _mm256_set1_epi16(NEG_LIMIT);
-    __m256i vPosLimit = _mm256_set1_epi16(POS_LIMIT);
-    __m256i vSaturationCheckMin = vPosLimit;
-    __m256i vSaturationCheckMax = vNegLimit;
-    __m256i vNegInf = _mm256_set1_epi16(NEG_LIMIT);
-    __m256i vOpen = _mm256_set1_epi16(open);
-    __m256i vGap  = _mm256_set1_epi16(gap);
-    __m256i vZero = _mm256_set1_epi16(0);
-    __m256i vOne = _mm256_set1_epi16(1);
-    __m256i vN = _mm256_set1_epi16(N);
-    __m256i vGapN = s1_beg ? _mm256_set1_epi16(0) : _mm256_set1_epi16(gap*N);
-    __m256i vNegOne = _mm256_set1_epi16(-1);
-    __m256i vI = _mm256_set_epi16(0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15);
-    __m256i vJreset = _mm256_set_epi16(0,-1,-2,-3,-4,-5,-6,-7,-8,-9,-10,-11,-12,-13,-14,-15);
-    __m256i vMaxHRow = vNegInf;
-    __m256i vMaxMRow = vNegInf;
-    __m256i vMaxSRow = vNegInf;
-    __m256i vMaxLRow = vNegInf;
-    __m256i vMaxHCol = vNegInf;
-    __m256i vMaxMCol = vNegInf;
-    __m256i vMaxSCol = vNegInf;
-    __m256i vMaxLCol = vNegInf;
-    __m256i vLastValH = vNegInf;
-    __m256i vLastValM = vNegInf;
-    __m256i vLastValS = vNegInf;
-    __m256i vLastValL = vNegInf;
-    __m256i vEndI = vNegInf;
-    __m256i vEndJ = vNegInf;
-    __m256i vILimit = _mm256_set1_epi16(s1Len);
-    __m256i vILimit1 = _mm256_sub_epi16(vILimit, vOne);
-    __m256i vJLimit = _mm256_set1_epi16(s2Len);
-    __m256i vJLimit1 = _mm256_sub_epi16(vJLimit, vOne);
-    __m256i vIBoundary = s1_beg ? _mm256_set1_epi16(0) : _mm256_set_epi16(
+    int32_t end_query = 0;
+    int32_t end_ref = 0;
+    int16_t NEG_LIMIT = 0;
+    int16_t POS_LIMIT = 0;
+    int16_t score = 0;
+    int16_t matches = 0;
+    int16_t similar = 0;
+    int16_t length = 0;
+    __m256i vNegLimit;
+    __m256i vPosLimit;
+    __m256i vSaturationCheckMin;
+    __m256i vSaturationCheckMax;
+    __m256i vNegInf;
+    __m256i vOpen;
+    __m256i vGap;
+    __m256i vZero;
+    __m256i vOne;
+    __m256i vN;
+    __m256i vGapN;
+    __m256i vNegOne;
+    __m256i vI;
+    __m256i vJreset;
+    __m256i vMaxHRow;
+    __m256i vMaxMRow;
+    __m256i vMaxSRow;
+    __m256i vMaxLRow;
+    __m256i vMaxHCol;
+    __m256i vMaxMCol;
+    __m256i vMaxSCol;
+    __m256i vMaxLCol;
+    __m256i vLastValH;
+    __m256i vLastValM;
+    __m256i vLastValS;
+    __m256i vLastValL;
+    __m256i vEndI;
+    __m256i vEndJ;
+    __m256i vILimit;
+    __m256i vILimit1;
+    __m256i vJLimit;
+    __m256i vJLimit1;
+    __m256i vIBoundary;
+
+    /* validate inputs */
+    PARASAIL_CHECK_NULL(_s2);
+    PARASAIL_CHECK_GT0(s2Len);
+    PARASAIL_CHECK_GE0(open);
+    PARASAIL_CHECK_GE0(gap);
+    PARASAIL_CHECK_NULL(matrix);
+    if (matrix->type == PARASAIL_MATRIX_TYPE_PSSM) {
+        PARASAIL_CHECK_NULL_PSSM_STATS(_s1);
+    }
+    else {
+        PARASAIL_CHECK_NULL(_s1);
+        PARASAIL_CHECK_GT0(_s1Len);
+    }
+
+    /* initialize stack variables */
+    N = 16; /* number of values in vector */
+    PAD = N-1;
+    PAD2 = PAD*2;
+    s1Len = matrix->type == PARASAIL_MATRIX_TYPE_SQUARE ? _s1Len : matrix->length;
+    s1Len_PAD = s1Len+PAD;
+    s2Len_PAD = s2Len+PAD;
+    i = 0;
+    j = 0;
+    end_query = s1Len-1;
+    end_ref = s2Len-1;
+    NEG_LIMIT = (-open < matrix->min ? INT16_MIN + open : INT16_MIN - matrix->min) + 1;
+    POS_LIMIT = INT16_MAX - matrix->max - 1;
+    score = NEG_LIMIT;
+    matches = NEG_LIMIT;
+    similar = NEG_LIMIT;
+    length = NEG_LIMIT;
+    vNegLimit = _mm256_set1_epi16(NEG_LIMIT);
+    vPosLimit = _mm256_set1_epi16(POS_LIMIT);
+    vSaturationCheckMin = vPosLimit;
+    vSaturationCheckMax = vNegLimit;
+    vNegInf = _mm256_set1_epi16(NEG_LIMIT);
+    vOpen = _mm256_set1_epi16(open);
+    vGap  = _mm256_set1_epi16(gap);
+    vZero = _mm256_set1_epi16(0);
+    vOne = _mm256_set1_epi16(1);
+    vN = _mm256_set1_epi16(N);
+    vGapN = s1_beg ? _mm256_set1_epi16(0) : _mm256_set1_epi16(gap*N);
+    vNegOne = _mm256_set1_epi16(-1);
+    vI = _mm256_set_epi16(0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15);
+    vJreset = _mm256_set_epi16(0,-1,-2,-3,-4,-5,-6,-7,-8,-9,-10,-11,-12,-13,-14,-15);
+    vMaxHRow = vNegInf;
+    vMaxMRow = vNegInf;
+    vMaxSRow = vNegInf;
+    vMaxLRow = vNegInf;
+    vMaxHCol = vNegInf;
+    vMaxMCol = vNegInf;
+    vMaxSCol = vNegInf;
+    vMaxLCol = vNegInf;
+    vLastValH = vNegInf;
+    vLastValM = vNegInf;
+    vLastValS = vNegInf;
+    vLastValL = vNegInf;
+    vEndI = vNegInf;
+    vEndJ = vNegInf;
+    vILimit = _mm256_set1_epi16(s1Len);
+    vILimit1 = _mm256_subs_epi16(vILimit, vOne);
+    vJLimit = _mm256_set1_epi16(s2Len);
+    vJLimit1 = _mm256_subs_epi16(vJLimit, vOne);
+    vIBoundary = s1_beg ? _mm256_set1_epi16(0) : _mm256_set_epi16(
             -open-0*gap,
             -open-1*gap,
             -open-2*gap,
@@ -326,6 +383,66 @@ parasail_result_t* FNAME(
             -open-14*gap,
             -open-15*gap);
 
+    /* initialize result */
+#ifdef PARASAIL_TABLE
+    result = parasail_result_new_table3(s1Len, s2Len);
+#else
+#ifdef PARASAIL_ROWCOL
+    result = parasail_result_new_rowcol3(s1Len, s2Len);
+#else
+    result = parasail_result_new_stats();
+#endif
+#endif
+    if (!result) return NULL;
+
+    /* set known flags */
+    result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_DIAG
+        | PARASAIL_FLAG_STATS
+        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_16;
+    result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
+    result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
+    result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
+    result->flag |= s2_end ? PARASAIL_FLAG_SG_S2_END : 0;
+#ifdef PARASAIL_TABLE
+    result->flag |= PARASAIL_FLAG_TABLE;
+#endif
+#ifdef PARASAIL_ROWCOL
+    result->flag |= PARASAIL_FLAG_ROWCOL;
+#endif
+
+    /* initialize heap variables */
+    s1     = parasail_memalign_int16_t(32, s1Len+PAD);
+    s2B    = parasail_memalign_int16_t(32, s2Len+PAD2);
+    _H_pr  = parasail_memalign_int16_t(32, s2Len+PAD2);
+    _HM_pr = parasail_memalign_int16_t(32, s2Len+PAD2);
+    _HS_pr = parasail_memalign_int16_t(32, s2Len+PAD2);
+    _HL_pr = parasail_memalign_int16_t(32, s2Len+PAD2);
+    _F_pr  = parasail_memalign_int16_t(32, s2Len+PAD2);
+    _FM_pr = parasail_memalign_int16_t(32, s2Len+PAD2);
+    _FS_pr = parasail_memalign_int16_t(32, s2Len+PAD2);
+    _FL_pr = parasail_memalign_int16_t(32, s2Len+PAD2);
+    s2 = s2B+PAD; /* will allow later for negative indices */
+    H_pr = _H_pr+PAD;
+    HM_pr = _HM_pr+PAD;
+    HS_pr = _HS_pr+PAD;
+    HL_pr = _HL_pr+PAD;
+    F_pr = _F_pr+PAD;
+    FM_pr = _FM_pr+PAD;
+    FS_pr = _FS_pr+PAD;
+    FL_pr = _FL_pr+PAD;
+
+    /* validate heap variables */
+    if (!s1) return NULL;
+    if (!s2B) return NULL;
+    if (!_H_pr) return NULL;
+    if (!_HM_pr) return NULL;
+    if (!_HS_pr) return NULL;
+    if (!_HL_pr) return NULL;
+    if (!_F_pr) return NULL;
+    if (!_FM_pr) return NULL;
+    if (!_FS_pr) return NULL;
+    if (!_FL_pr) return NULL;
+
     /* convert _s1 from char to int in range 0-23 */
     for (i=0; i<s1Len; ++i) {
         s1[i] = matrix->mapper[(unsigned char)_s1[i]];
@@ -355,7 +472,7 @@ parasail_result_t* FNAME(
             HM_pr[j] = 0;
             HS_pr[j] = 0;
             HL_pr[j] = 0;
-            F_pr[j] = NEG_INF;
+            F_pr[j] = NEG_LIMIT;
             FM_pr[j] = 0;
             FS_pr[j] = 0;
             FL_pr[j] = 0;
@@ -367,7 +484,7 @@ parasail_result_t* FNAME(
             HM_pr[j] = 0;
             HS_pr[j] = 0;
             HL_pr[j] = 0;
-            F_pr[j] = NEG_INF;
+            F_pr[j] = NEG_LIMIT;
             FM_pr[j] = 0;
             FS_pr[j] = 0;
             FL_pr[j] = 0;
@@ -440,22 +557,22 @@ parasail_result_t* FNAME(
                 s1[i+14],
                 s1[i+15]);
         __m256i vs2 = vNegInf;
-        const int * const restrict matrow0 = &matrix->matrix[matrix->size*s1[i+0]];
-        const int * const restrict matrow1 = &matrix->matrix[matrix->size*s1[i+1]];
-        const int * const restrict matrow2 = &matrix->matrix[matrix->size*s1[i+2]];
-        const int * const restrict matrow3 = &matrix->matrix[matrix->size*s1[i+3]];
-        const int * const restrict matrow4 = &matrix->matrix[matrix->size*s1[i+4]];
-        const int * const restrict matrow5 = &matrix->matrix[matrix->size*s1[i+5]];
-        const int * const restrict matrow6 = &matrix->matrix[matrix->size*s1[i+6]];
-        const int * const restrict matrow7 = &matrix->matrix[matrix->size*s1[i+7]];
-        const int * const restrict matrow8 = &matrix->matrix[matrix->size*s1[i+8]];
-        const int * const restrict matrow9 = &matrix->matrix[matrix->size*s1[i+9]];
-        const int * const restrict matrow10 = &matrix->matrix[matrix->size*s1[i+10]];
-        const int * const restrict matrow11 = &matrix->matrix[matrix->size*s1[i+11]];
-        const int * const restrict matrow12 = &matrix->matrix[matrix->size*s1[i+12]];
-        const int * const restrict matrow13 = &matrix->matrix[matrix->size*s1[i+13]];
-        const int * const restrict matrow14 = &matrix->matrix[matrix->size*s1[i+14]];
-        const int * const restrict matrow15 = &matrix->matrix[matrix->size*s1[i+15]];
+        const int * const restrict matrow0 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+0] : ((i+0 >= s1Len) ? s1Len-1 : i+0))];
+        const int * const restrict matrow1 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+1] : ((i+1 >= s1Len) ? s1Len-1 : i+1))];
+        const int * const restrict matrow2 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+2] : ((i+2 >= s1Len) ? s1Len-1 : i+2))];
+        const int * const restrict matrow3 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+3] : ((i+3 >= s1Len) ? s1Len-1 : i+3))];
+        const int * const restrict matrow4 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+4] : ((i+4 >= s1Len) ? s1Len-1 : i+4))];
+        const int * const restrict matrow5 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+5] : ((i+5 >= s1Len) ? s1Len-1 : i+5))];
+        const int * const restrict matrow6 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+6] : ((i+6 >= s1Len) ? s1Len-1 : i+6))];
+        const int * const restrict matrow7 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+7] : ((i+7 >= s1Len) ? s1Len-1 : i+7))];
+        const int * const restrict matrow8 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+8] : ((i+8 >= s1Len) ? s1Len-1 : i+8))];
+        const int * const restrict matrow9 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+9] : ((i+9 >= s1Len) ? s1Len-1 : i+9))];
+        const int * const restrict matrow10 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+10] : ((i+10 >= s1Len) ? s1Len-1 : i+10))];
+        const int * const restrict matrow11 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+11] : ((i+11 >= s1Len) ? s1Len-1 : i+11))];
+        const int * const restrict matrow12 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+12] : ((i+12 >= s1Len) ? s1Len-1 : i+12))];
+        const int * const restrict matrow13 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+13] : ((i+13 >= s1Len) ? s1Len-1 : i+13))];
+        const int * const restrict matrow14 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+14] : ((i+14 >= s1Len) ? s1Len-1 : i+14))];
+        const int * const restrict matrow15 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+15] : ((i+15 >= s1Len) ? s1Len-1 : i+15))];
         __m256i vIltLimit = _mm256_cmplt_epi16_rpl(vI, vILimit);
         __m256i vIeqLimit1 = _mm256_cmpeq_epi16(vI, vILimit1);
         vNH = _mm256_srli_si256_rpl(vNH, 2);
@@ -486,22 +603,22 @@ parasail_result_t* FNAME(
             vFS = _mm256_insert_epi16_rpl(vFS, FS_pr[j], 15);
             vFL = _mm256_srli_si256_rpl(vFL, 2);
             vFL = _mm256_insert_epi16_rpl(vFL, FL_pr[j], 15);
-            vF_opn = _mm256_sub_epi16(vNH, vOpen);
-            vF_ext = _mm256_sub_epi16(vF, vGap);
+            vF_opn = _mm256_subs_epi16(vNH, vOpen);
+            vF_ext = _mm256_subs_epi16(vF, vGap);
             vF = _mm256_max_epi16(vF_opn, vF_ext);
             case1 = _mm256_cmpgt_epi16(vF_opn, vF_ext);
             vFM = _mm256_blendv_epi8(vFM, vNM, case1);
             vFS = _mm256_blendv_epi8(vFS, vNS, case1);
             vFL = _mm256_blendv_epi8(vFL, vNL, case1);
-            vFL = _mm256_add_epi16(vFL, vOne);
-            vE_opn = _mm256_sub_epi16(vWH, vOpen);
-            vE_ext = _mm256_sub_epi16(vE, vGap);
+            vFL = _mm256_adds_epi16(vFL, vOne);
+            vE_opn = _mm256_subs_epi16(vWH, vOpen);
+            vE_ext = _mm256_subs_epi16(vE, vGap);
             vE = _mm256_max_epi16(vE_opn, vE_ext);
             case1 = _mm256_cmpgt_epi16(vE_opn, vE_ext);
             vEM = _mm256_blendv_epi8(vEM, vWM, case1);
             vES = _mm256_blendv_epi8(vES, vWS, case1);
             vEL = _mm256_blendv_epi8(vEL, vWL, case1);
-            vEL = _mm256_add_epi16(vEL, vOne);
+            vEL = _mm256_adds_epi16(vEL, vOne);
             vs2 = _mm256_srli_si256_rpl(vs2, 2);
             vs2 = _mm256_insert_epi16_rpl(vs2, s2[j], 15);
             vMat = _mm256_set_epi16(
@@ -522,28 +639,28 @@ parasail_result_t* FNAME(
                     matrow14[s2[j-14]],
                     matrow15[s2[j-15]]
                     );
-            vNWH = _mm256_add_epi16(vNWH, vMat);
+            vNWH = _mm256_adds_epi16(vNWH, vMat);
             vWH = _mm256_max_epi16(vNWH, vE);
             vWH = _mm256_max_epi16(vWH, vF);
             case1 = _mm256_cmpeq_epi16(vWH, vNWH);
             case2 = _mm256_cmpeq_epi16(vWH, vF);
             vWM = _mm256_blendv_epi8(
                     _mm256_blendv_epi8(vEM, vFM, case2),
-                    _mm256_add_epi16(vNWM,
+                    _mm256_adds_epi16(vNWM,
                         _mm256_and_si256(
                             _mm256_cmpeq_epi16(vs1,vs2),
                             vOne)),
                     case1);
             vWS = _mm256_blendv_epi8(
                     _mm256_blendv_epi8(vES, vFS, case2),
-                    _mm256_add_epi16(vNWS,
+                    _mm256_adds_epi16(vNWS,
                         _mm256_and_si256(
                             _mm256_cmpgt_epi16(vMat,vZero),
                             vOne)),
                     case1);
             vWL = _mm256_blendv_epi8(
                     _mm256_blendv_epi8(vEL, vFL, case2),
-                    _mm256_add_epi16(vNWL, vOne), case1);
+                    _mm256_adds_epi16(vNWL, vOne), case1);
             /* as minor diagonal vector passes across the j=-1 boundary,
              * assign the appropriate boundary conditions */
             {
@@ -557,13 +674,15 @@ parasail_result_t* FNAME(
                 vES = _mm256_andnot_si256(cond, vES);
                 vEL = _mm256_andnot_si256(cond, vEL);
             }
-            vSaturationCheckMin = _mm256_min_epi16(vSaturationCheckMin, vWH);
-            vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vWH);
-            vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vWM);
-            vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vWS);
-            vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vWL);
-            vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vWL);
-            vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vJ);
+            /* cannot start checking sat until after J clears boundary */
+            if (j > PAD) {
+                vSaturationCheckMin = _mm256_min_epi16(vSaturationCheckMin, vWH);
+                vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vWH);
+                vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vWM);
+                vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vWS);
+                vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vWL);
+                vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vWL);
+            }
 #ifdef PARASAIL_TABLE
             arr_store_si256(result->stats->tables->score_table, vWH, i, s1Len, j, s2Len);
             arr_store_si256(result->stats->tables->matches_table, vWM, i, s1Len, j, s2Len);
@@ -613,27 +732,26 @@ parasail_result_t* FNAME(
                 vEndI = _mm256_blendv_epi8(vEndI, vI, cond_all_col);
                 vEndJ = _mm256_blendv_epi8(vEndJ, vJ, cond_all_row);
             }
-            vJ = _mm256_add_epi16(vJ, vOne);
+            vJ = _mm256_adds_epi16(vJ, vOne);
         }
-        vI = _mm256_add_epi16(vI, vN);
-        vIBoundary = _mm256_sub_epi16(vIBoundary, vGapN);
-        vSaturationCheckMax = _mm256_max_epi16(vSaturationCheckMax, vI);
+        vI = _mm256_adds_epi16(vI, vN);
+        vIBoundary = _mm256_subs_epi16(vIBoundary, vGapN);
     }
 
     /* alignment ending position */
     {
-        int16_t max_rowh = NEG_INF;
-        int16_t max_rowm = NEG_INF;
-        int16_t max_rows = NEG_INF;
-        int16_t max_rowl = NEG_INF;
-        int16_t max_colh = NEG_INF;
-        int16_t max_colm = NEG_INF;
-        int16_t max_cols = NEG_INF;
-        int16_t max_coll = NEG_INF;
-        int16_t last_valh = NEG_INF;
-        int16_t last_valm = NEG_INF;
-        int16_t last_vals = NEG_INF;
-        int16_t last_vall = NEG_INF;
+        int16_t max_rowh = NEG_LIMIT;
+        int16_t max_rowm = NEG_LIMIT;
+        int16_t max_rows = NEG_LIMIT;
+        int16_t max_rowl = NEG_LIMIT;
+        int16_t max_colh = NEG_LIMIT;
+        int16_t max_colm = NEG_LIMIT;
+        int16_t max_cols = NEG_LIMIT;
+        int16_t max_coll = NEG_LIMIT;
+        int16_t last_valh = NEG_LIMIT;
+        int16_t last_valm = NEG_LIMIT;
+        int16_t last_vals = NEG_LIMIT;
+        int16_t last_vall = NEG_LIMIT;
         int16_t *rh = (int16_t*)&vMaxHRow;
         int16_t *rm = (int16_t*)&vMaxMRow;
         int16_t *rs = (int16_t*)&vMaxSRow;
@@ -729,19 +847,6 @@ parasail_result_t* FNAME(
     result->stats->matches = matches;
     result->stats->similar = similar;
     result->stats->length = length;
-    result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_DIAG
-        | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_16;
-    result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
-    result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
-    result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
-    result->flag |= s2_end ? PARASAIL_FLAG_SG_S2_END : 0;
-#ifdef PARASAIL_TABLE
-    result->flag |= PARASAIL_FLAG_TABLE;
-#endif
-#ifdef PARASAIL_ROWCOL
-    result->flag |= PARASAIL_FLAG_ROWCOL;
-#endif
 
     parasail_free(_FL_pr);
     parasail_free(_FS_pr);
