@@ -11,8 +11,8 @@
 #include <stdlib.h>
 
 #define SG_STATS
-#define SG_SUFFIX _striped_sse41_128_16
-#define SG_SUFFIX_PROF _striped_profile_sse41_128_16
+#define SG_SUFFIX _striped_sse41_128_32
+#define SG_SUFFIX_PROF _striped_profile_sse41_128_32
 #include "sg_helper.h"
 
 #if defined(_MSC_VER)
@@ -39,14 +39,10 @@ static inline void arr_store_si128(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*(0*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 0);
-    array[1LL*(1*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 1);
-    array[1LL*(2*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 2);
-    array[1LL*(3*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 3);
-    array[1LL*(4*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 4);
-    array[1LL*(5*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 5);
-    array[1LL*(6*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 6);
-    array[1LL*(7*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 7);
+    array[1LL*(0*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32(vH, 0);
+    array[1LL*(1*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32(vH, 1);
+    array[1LL*(2*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32(vH, 2);
+    array[1LL*(3*seglen+t)*dlen + d] = (int32_t)_mm_extract_epi32(vH, 3);
 }
 #endif
 
@@ -57,27 +53,23 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[0*seglen+t] = (int16_t)_mm_extract_epi16(vH, 0);
-    col[1*seglen+t] = (int16_t)_mm_extract_epi16(vH, 1);
-    col[2*seglen+t] = (int16_t)_mm_extract_epi16(vH, 2);
-    col[3*seglen+t] = (int16_t)_mm_extract_epi16(vH, 3);
-    col[4*seglen+t] = (int16_t)_mm_extract_epi16(vH, 4);
-    col[5*seglen+t] = (int16_t)_mm_extract_epi16(vH, 5);
-    col[6*seglen+t] = (int16_t)_mm_extract_epi16(vH, 6);
-    col[7*seglen+t] = (int16_t)_mm_extract_epi16(vH, 7);
+    col[0*seglen+t] = (int32_t)_mm_extract_epi32(vH, 0);
+    col[1*seglen+t] = (int32_t)_mm_extract_epi32(vH, 1);
+    col[2*seglen+t] = (int32_t)_mm_extract_epi32(vH, 2);
+    col[3*seglen+t] = (int32_t)_mm_extract_epi32(vH, 3);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sg_flags_stats_table_striped_sse41_128_16
-#define PNAME parasail_sg_flags_stats_table_striped_profile_sse41_128_16
+#define FNAME parasail_sg_flags_stats_table_striped_sse41_128_32
+#define PNAME parasail_sg_flags_stats_table_striped_profile_sse41_128_32
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sg_flags_stats_rowcol_striped_sse41_128_16
-#define PNAME parasail_sg_flags_stats_rowcol_striped_profile_sse41_128_16
+#define FNAME parasail_sg_flags_stats_rowcol_striped_sse41_128_32
+#define PNAME parasail_sg_flags_stats_rowcol_striped_profile_sse41_128_32
 #else
-#define FNAME parasail_sg_flags_stats_striped_sse41_128_16
-#define PNAME parasail_sg_flags_stats_striped_profile_sse41_128_16
+#define FNAME parasail_sg_flags_stats_striped_sse41_128_32
+#define PNAME parasail_sg_flags_stats_striped_profile_sse41_128_32
 #endif
 #endif
 
@@ -87,9 +79,31 @@ parasail_result_t* FNAME(
         const int open, const int gap, const parasail_matrix_t *matrix,
         int s1_beg, int s1_end, int s2_beg, int s2_end)
 {
-    parasail_profile_t *profile = parasail_profile_create_stats_sse_128_16(s1, s1Len, matrix);
-    parasail_result_t *result = PNAME(profile, s2, s2Len, open, gap, s1_beg, s1_end, s2_beg, s2_end);
+    /* declare local variables */
+    parasail_profile_t *profile = NULL;
+    parasail_result_t *result = NULL;
+
+    /* validate inputs */
+    PARASAIL_CHECK_NULL(s2);
+    PARASAIL_CHECK_GT0(s2Len);
+    PARASAIL_CHECK_GE0(open);
+    PARASAIL_CHECK_GE0(gap);
+    PARASAIL_CHECK_NULL(matrix);
+    if (matrix->type == PARASAIL_MATRIX_TYPE_PSSM) {
+        PARASAIL_CHECK_NULL_PSSM_STATS(s1);
+    }
+    else {
+        PARASAIL_CHECK_NULL(s1);
+        PARASAIL_CHECK_GT0(s1Len);
+    }
+
+    /* initialize local variables */
+    profile = parasail_profile_create_stats_sse_128_32(s1, s1Len, matrix);
+    if (!profile) return NULL;
+    result = PNAME(profile, s2, s2Len, open, gap, s1_beg, s1_end, s2_beg, s2_end);
+
     parasail_profile_free(profile);
+
     return result;
 }
 
@@ -99,63 +113,157 @@ parasail_result_t* PNAME(
         const int open, const int gap,
         int s1_beg, int s1_end, int s2_beg, int s2_end)
 {
+    /* declare local variables */
     int32_t i = 0;
     int32_t j = 0;
     int32_t k = 0;
-    const int s1Len = profile->s1Len;
-    int32_t end_query = s1Len-1;
-    int32_t end_ref = s2Len-1;
-    const parasail_matrix_t *matrix = profile->matrix;
-    const int32_t segWidth = 8; /* number of values in vector unit */
-    const int32_t segLen = (s1Len + segWidth - 1) / segWidth;
-    const int32_t offset = (s1Len - 1) % segLen;
-    const int32_t position = (segWidth - 1) - (s1Len - 1) / segLen;
-    __m128i* const restrict vProfile  = (__m128i*)profile->profile16.score;
-    __m128i* const restrict vProfileM = (__m128i*)profile->profile16.matches;
-    __m128i* const restrict vProfileS = (__m128i*)profile->profile16.similar;
-    __m128i* restrict pvHStore        = parasail_memalign___m128i(16, segLen);
-    __m128i* restrict pvHLoad         = parasail_memalign___m128i(16, segLen);
-    __m128i* restrict pvHMStore       = parasail_memalign___m128i(16, segLen);
-    __m128i* restrict pvHMLoad        = parasail_memalign___m128i(16, segLen);
-    __m128i* restrict pvHSStore       = parasail_memalign___m128i(16, segLen);
-    __m128i* restrict pvHSLoad        = parasail_memalign___m128i(16, segLen);
-    __m128i* restrict pvHLStore       = parasail_memalign___m128i(16, segLen);
-    __m128i* restrict pvHLLoad        = parasail_memalign___m128i(16, segLen);
-    __m128i* const restrict pvE       = parasail_memalign___m128i(16, segLen);
-    __m128i* const restrict pvEM      = parasail_memalign___m128i(16, segLen);
-    __m128i* const restrict pvES      = parasail_memalign___m128i(16, segLen);
-    __m128i* const restrict pvEL      = parasail_memalign___m128i(16, segLen);
-    int16_t* const restrict boundary = parasail_memalign_int16_t(16, s2Len+1);
-    const __m128i vGapO = _mm_set1_epi16(open);
-    const __m128i vGapE = _mm_set1_epi16(gap);
-    const int16_t NEG_LIMIT = (-open < matrix->min ?
-        INT16_MIN + open : INT16_MIN - matrix->min) + 1;
-    const int16_t POS_LIMIT = INT16_MAX - matrix->max - 1;
-    const __m128i vZero = _mm_setzero_si128();
-    const __m128i vOne = _mm_set1_epi16(1);
-    int16_t score = NEG_LIMIT;
-    int16_t matches = 0;
-    int16_t similar = 0;
-    int16_t length = 0;
-    __m128i vNegLimit = _mm_set1_epi16(NEG_LIMIT);
-    __m128i vPosLimit = _mm_set1_epi16(POS_LIMIT);
-    __m128i vSaturationCheckMin = vPosLimit;
-    __m128i vSaturationCheckMax = vNegLimit;
-    __m128i vMaxH = vNegLimit;
-    __m128i vMaxHM = vNegLimit;
-    __m128i vMaxHS = vNegLimit;
-    __m128i vMaxHL = vNegLimit;
-    __m128i vPosMask = _mm_cmpeq_epi16(_mm_set1_epi16(position),
-            _mm_set_epi16(0,1,2,3,4,5,6,7));
+    int32_t s1Len = 0;
+    int32_t end_query = 0;
+    int32_t end_ref = 0;
+    const parasail_matrix_t *matrix = NULL;
+    int32_t segWidth = 0;
+    int32_t segLen = 0;
+    int32_t offset = 0;
+    int32_t position = 0;
+    __m128i* restrict vProfile = NULL;
+    __m128i* restrict vProfileM = NULL;
+    __m128i* restrict vProfileS = NULL;
+    __m128i* restrict pvHStore = NULL;
+    __m128i* restrict pvHLoad = NULL;
+    __m128i* restrict pvHMStore = NULL;
+    __m128i* restrict pvHMLoad = NULL;
+    __m128i* restrict pvHSStore = NULL;
+    __m128i* restrict pvHSLoad = NULL;
+    __m128i* restrict pvHLStore = NULL;
+    __m128i* restrict pvHLLoad = NULL;
+    __m128i* restrict pvE = NULL;
+    __m128i* restrict pvEM = NULL;
+    __m128i* restrict pvES = NULL;
+    __m128i* restrict pvEL = NULL;
+    int32_t* restrict boundary = NULL;
+    __m128i vGapO;
+    __m128i vGapE;
+    int32_t NEG_LIMIT = 0;
+    int32_t POS_LIMIT = 0;
+    __m128i vZero;
+    __m128i vOne;
+    int32_t score = 0;
+    int32_t matches = 0;
+    int32_t similar = 0;
+    int32_t length = 0;
+    __m128i vNegLimit;
+    __m128i vPosLimit;
+    __m128i vSaturationCheckMin;
+    __m128i vSaturationCheckMax;
+    __m128i vMaxH;
+    __m128i vMaxHM;
+    __m128i vMaxHS;
+    __m128i vMaxHL;
+    __m128i vPosMask;
+    parasail_result_t *result = NULL;
+
+    /* validate inputs */
+    PARASAIL_CHECK_NULL(profile);
+    PARASAIL_CHECK_NULL(profile->profile32.score);
+    PARASAIL_CHECK_NULL(profile->matrix);
+    PARASAIL_CHECK_GT0(profile->s1Len);
+    PARASAIL_CHECK_NULL(s2);
+    PARASAIL_CHECK_GT0(s2Len);
+    PARASAIL_CHECK_GE0(open);
+    PARASAIL_CHECK_GE0(gap);
+
+    /* initialize stack variables */
+    i = 0;
+    j = 0;
+    k = 0;
+    s1Len = profile->s1Len;
+    end_query = s1Len-1;
+    end_ref = s2Len-1;
+    matrix = profile->matrix;
+    segWidth = 4; /* number of values in vector unit */
+    segLen = (s1Len + segWidth - 1) / segWidth;
+    offset = (s1Len - 1) % segLen;
+    position = (segWidth - 1) - (s1Len - 1) / segLen;
+    vProfile  = (__m128i*)profile->profile32.score;
+    vProfileM = (__m128i*)profile->profile32.matches;
+    vProfileS = (__m128i*)profile->profile32.similar;
+    vGapO = _mm_set1_epi32(open);
+    vGapE = _mm_set1_epi32(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT32_MIN + open : INT32_MIN - matrix->min) + 1;
+    POS_LIMIT = INT32_MAX - matrix->max - 1;
+    vZero = _mm_setzero_si128();
+    vOne = _mm_set1_epi32(1);
+    score = NEG_LIMIT;
+    matches = 0;
+    similar = 0;
+    length = 0;
+    vNegLimit = _mm_set1_epi32(NEG_LIMIT);
+    vPosLimit = _mm_set1_epi32(POS_LIMIT);
+    vSaturationCheckMin = vPosLimit;
+    vSaturationCheckMax = vNegLimit;
+    vMaxH = vNegLimit;
+    vMaxHM = vNegLimit;
+    vMaxHS = vNegLimit;
+    vMaxHL = vNegLimit;
+    vPosMask = _mm_cmpeq_epi32(_mm_set1_epi32(position),
+            _mm_set_epi32(0,1,2,3));
+
+    /* initialize result */
 #ifdef PARASAIL_TABLE
-    parasail_result_t *result = parasail_result_new_table3(segLen*segWidth, s2Len);
+    result = parasail_result_new_table3(segLen*segWidth, s2Len);
 #else
 #ifdef PARASAIL_ROWCOL
-    parasail_result_t *result = parasail_result_new_rowcol3(segLen*segWidth, s2Len);
+    result = parasail_result_new_rowcol3(segLen*segWidth, s2Len);
 #else
-    parasail_result_t *result = parasail_result_new_stats();
+    result = parasail_result_new_stats();
 #endif
 #endif
+    if (!result) return NULL;
+
+    /* set known flags */
+    result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_STRIPED
+        | PARASAIL_FLAG_STATS
+        | PARASAIL_FLAG_BITS_32 | PARASAIL_FLAG_LANES_4;
+    result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
+    result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
+    result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
+    result->flag |= s2_end ? PARASAIL_FLAG_SG_S2_END : 0;
+#ifdef PARASAIL_TABLE
+    result->flag |= PARASAIL_FLAG_TABLE;
+#endif
+#ifdef PARASAIL_ROWCOL
+    result->flag |= PARASAIL_FLAG_ROWCOL;
+#endif
+
+    /* initialize heap variables */
+    pvHStore  = parasail_memalign___m128i(16, segLen);
+    pvHLoad   = parasail_memalign___m128i(16, segLen);
+    pvHMStore = parasail_memalign___m128i(16, segLen);
+    pvHMLoad  = parasail_memalign___m128i(16, segLen);
+    pvHSStore = parasail_memalign___m128i(16, segLen);
+    pvHSLoad  = parasail_memalign___m128i(16, segLen);
+    pvHLStore = parasail_memalign___m128i(16, segLen);
+    pvHLLoad  = parasail_memalign___m128i(16, segLen);
+    pvE       = parasail_memalign___m128i(16, segLen);
+    pvEM      = parasail_memalign___m128i(16, segLen);
+    pvES      = parasail_memalign___m128i(16, segLen);
+    pvEL      = parasail_memalign___m128i(16, segLen);
+    boundary  = parasail_memalign_int32_t(16, s2Len+1);
+
+    /* validate heap variables */
+    if (!pvHStore) return NULL;
+    if (!pvHLoad) return NULL;
+    if (!pvHMStore) return NULL;
+    if (!pvHMLoad) return NULL;
+    if (!pvHSStore) return NULL;
+    if (!pvHSLoad) return NULL;
+    if (!pvHLStore) return NULL;
+    if (!pvHLLoad) return NULL;
+    if (!pvE) return NULL;
+    if (!pvEM) return NULL;
+    if (!pvES) return NULL;
+    if (!pvEL) return NULL;
+    if (!boundary) return NULL;
 
     parasail_memset___m128i(pvHMStore, vZero, segLen);
     parasail_memset___m128i(pvHSStore, vZero, segLen);
@@ -169,13 +277,13 @@ parasail_result_t* PNAME(
         int32_t index = 0;
         for (i=0; i<segLen; ++i) {
             int32_t segNum = 0;
-            __m128i_16_t h;
-            __m128i_16_t e;
+            __m128i_32_t h;
+            __m128i_32_t e;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = s1_beg ? 0 : (-open-gap*(segNum*segLen+i));
-                h.v[segNum] = tmp < INT16_MIN ? INT16_MIN : tmp;
+                h.v[segNum] = tmp < INT32_MIN ? INT32_MIN : tmp;
                 tmp = tmp - open;
-                e.v[segNum] = tmp < INT16_MIN ? INT16_MIN : tmp;
+                e.v[segNum] = tmp < INT32_MIN ? INT32_MIN : tmp;
             }
             _mm_store_si128(&pvHStore[index], h.m);
             _mm_store_si128(&pvE[index], e.m);
@@ -188,7 +296,7 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = s2_beg ? 0 : (-open-gap*(i-1));
-            boundary[i] = tmp < INT16_MIN ? INT16_MIN : tmp;
+            boundary[i] = tmp < INT32_MIN ? INT32_MIN : tmp;
         }
     }
 
@@ -221,18 +329,18 @@ parasail_result_t* PNAME(
         vFS = vZero;
         vFL = vOne;
 
-        /* load final segment of pvHStore and shift left by 2 bytes */
+        /* load final segment of pvHStore and shift left by 4 bytes */
         vH = _mm_load_si128(&pvHStore[segLen - 1]);
         vHM = _mm_load_si128(&pvHMStore[segLen - 1]);
         vHS = _mm_load_si128(&pvHSStore[segLen - 1]);
         vHL = _mm_load_si128(&pvHLStore[segLen - 1]);
-        vH = _mm_slli_si128(vH, 2);
-        vHM = _mm_slli_si128(vHM, 2);
-        vHS = _mm_slli_si128(vHS, 2);
-        vHL = _mm_slli_si128(vHL, 2);
+        vH = _mm_slli_si128(vH, 4);
+        vHM = _mm_slli_si128(vHM, 4);
+        vHS = _mm_slli_si128(vHS, 4);
+        vHL = _mm_slli_si128(vHL, 4);
 
         /* insert upper boundary condition */
-        vH = _mm_insert_epi16(vH, boundary[j], 0);
+        vH = _mm_insert_epi32(vH, boundary[j], 0);
 
         /* Correct part of the vProfile */
         vP = vProfile + matrix->mapper[(unsigned char)s2[j]] * segLen;
@@ -256,58 +364,58 @@ parasail_result_t* PNAME(
             vEL = _mm_load_si128(pvEL+ i);
 
             /* Get max from vH, vE and vF. */
-            vH_dag = _mm_add_epi16(vH, _mm_load_si128(vP + i));
-            vH = _mm_max_epi16(vH_dag, vE);
-            vH = _mm_max_epi16(vH, vF);
+            vH_dag = _mm_add_epi32(vH, _mm_load_si128(vP + i));
+            vH = _mm_max_epi32(vH_dag, vE);
+            vH = _mm_max_epi32(vH, vF);
             /* Save vH values. */
             _mm_store_si128(pvHStore + i, vH);
 
-            case1 = _mm_cmpeq_epi16(vH, vH_dag);
-            case2 = _mm_cmpeq_epi16(vH, vF);
+            case1 = _mm_cmpeq_epi32(vH, vH_dag);
+            case2 = _mm_cmpeq_epi32(vH, vF);
 
             /* calculate vM */
             vHM = _mm_blendv_epi8(
                     _mm_blendv_epi8(vEM, vFM, case2),
-                    _mm_add_epi16(vHM, _mm_load_si128(vPM + i)),
+                    _mm_add_epi32(vHM, _mm_load_si128(vPM + i)),
                     case1);
             _mm_store_si128(pvHMStore + i, vHM);
 
             /* calculate vS */
             vHS = _mm_blendv_epi8(
                     _mm_blendv_epi8(vES, vFS, case2),
-                    _mm_add_epi16(vHS, _mm_load_si128(vPS + i)),
+                    _mm_add_epi32(vHS, _mm_load_si128(vPS + i)),
                     case1);
             _mm_store_si128(pvHSStore + i, vHS);
 
             /* calculate vL */
             vHL = _mm_blendv_epi8(
                     _mm_blendv_epi8(vEL, vFL, case2),
-                    _mm_add_epi16(vHL, vOne),
+                    _mm_add_epi32(vHL, vOne),
                     case1);
             _mm_store_si128(pvHLStore + i, vHL);
 
-            vSaturationCheckMin = _mm_min_epi16(vSaturationCheckMin, vH);
-            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vH);
-            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHM);
-            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHS);
-            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHL);
+            vSaturationCheckMin = _mm_min_epi32(vSaturationCheckMin, vH);
+            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vH);
+            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vHM);
+            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vHS);
+            vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
             arr_store_si128(result->stats->tables->similar_table, vHS, i, segLen, j, s2Len);
             arr_store_si128(result->stats->tables->length_table, vHL, i, segLen, j, s2Len);
             arr_store_si128(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
-            vEF_opn = _mm_sub_epi16(vH, vGapO);
+            vEF_opn = _mm_sub_epi32(vH, vGapO);
 
             /* Update vE value. */
-            vE_ext = _mm_sub_epi16(vE, vGapE);
-            vE = _mm_max_epi16(vEF_opn, vE_ext);
-            case1 = _mm_cmpgt_epi16(vEF_opn, vE_ext);
+            vE_ext = _mm_sub_epi32(vE, vGapE);
+            vE = _mm_max_epi32(vEF_opn, vE_ext);
+            case1 = _mm_cmpgt_epi32(vEF_opn, vE_ext);
             vEM = _mm_blendv_epi8(vEM, vHM, case1);
             vES = _mm_blendv_epi8(vES, vHS, case1);
             vEL = _mm_blendv_epi8(
-                    _mm_add_epi16(vEL, vOne),
-                    _mm_add_epi16(vHL, vOne),
+                    _mm_add_epi32(vEL, vOne),
+                    _mm_add_epi32(vHL, vOne),
                     case1);
             _mm_store_si128(pvE + i, vE);
             _mm_store_si128(pvEM + i, vEM);
@@ -315,14 +423,14 @@ parasail_result_t* PNAME(
             _mm_store_si128(pvEL + i, vEL);
 
             /* Update vF value. */
-            vF_ext = _mm_sub_epi16(vF, vGapE);
-            vF = _mm_max_epi16(vEF_opn, vF_ext);
-            case1 = _mm_cmpgt_epi16(vEF_opn, vF_ext);
+            vF_ext = _mm_sub_epi32(vF, vGapE);
+            vF = _mm_max_epi32(vEF_opn, vF_ext);
+            case1 = _mm_cmpgt_epi32(vEF_opn, vF_ext);
             vFM = _mm_blendv_epi8(vFM, vHM, case1);
             vFS = _mm_blendv_epi8(vFS, vHS, case1);
             vFL = _mm_blendv_epi8(
-                    _mm_add_epi16(vFL, vOne),
-                    _mm_add_epi16(vHL, vOne),
+                    _mm_add_epi32(vFL, vOne),
+                    _mm_add_epi32(vHL, vOne),
                     case1);
 
             /* Load the next vH. */
@@ -337,26 +445,26 @@ parasail_result_t* PNAME(
         for (k=0; k<segWidth; ++k) {
             __m128i vHp = _mm_load_si128(&pvHLoad[segLen - 1]);
             int64_t tmp = s2_beg ? -open : (boundary[j+1]-open);
-            int16_t tmp2 = tmp < INT16_MIN ? INT16_MIN : tmp;
-            vHp = _mm_slli_si128(vHp, 2);
-            vF = _mm_slli_si128(vF, 2);
-            vFM = _mm_slli_si128(vFM, 2);
-            vFS = _mm_slli_si128(vFS, 2);
-            vFL = _mm_slli_si128(vFL, 2);
-            vHp = _mm_insert_epi16(vHp, boundary[j], 0);
-            vF = _mm_insert_epi16(vF, tmp2, 0);
-            vFL = _mm_insert_epi16(vFL, 1, 0);
+            int32_t tmp2 = tmp < INT32_MIN ? INT32_MIN : tmp;
+            vHp = _mm_slli_si128(vHp, 4);
+            vF = _mm_slli_si128(vF, 4);
+            vFM = _mm_slli_si128(vFM, 4);
+            vFS = _mm_slli_si128(vFS, 4);
+            vFL = _mm_slli_si128(vFL, 4);
+            vHp = _mm_insert_epi32(vHp, boundary[j], 0);
+            vF = _mm_insert_epi32(vF, tmp2, 0);
+            vFL = _mm_insert_epi32(vFL, 1, 0);
             for (i=0; i<segLen; ++i) {
                 __m128i case1;
                 __m128i case2;
                 __m128i cond;
 
-                vHp = _mm_add_epi16(vHp, _mm_load_si128(vP + i));
+                vHp = _mm_add_epi32(vHp, _mm_load_si128(vP + i));
                 vH = _mm_load_si128(pvHStore + i);
-                vH = _mm_max_epi16(vH,vF);
+                vH = _mm_max_epi32(vH,vF);
                 _mm_store_si128(pvHStore + i, vH);
-                case1 = _mm_cmpeq_epi16(vH, vHp);
-                case2 = _mm_cmpeq_epi16(vH, vF);
+                case1 = _mm_cmpeq_epi32(vH, vHp);
+                case2 = _mm_cmpeq_epi32(vH, vF);
                 cond = _mm_andnot_si128(case1, case2);
 
                 /* calculate vM */
@@ -374,11 +482,11 @@ parasail_result_t* PNAME(
                 vHL = _mm_blendv_epi8(vHL, vFL, cond);
                 _mm_store_si128(pvHLStore + i, vHL);
 
-                vSaturationCheckMin = _mm_min_epi16(vSaturationCheckMin, vH);
-                vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vH);
-                vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHM);
-                vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHS);
-                vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHL);
+                vSaturationCheckMin = _mm_min_epi32(vSaturationCheckMin, vH);
+                vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vH);
+                vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vHM);
+                vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vHS);
+                vSaturationCheckMax = _mm_max_epi32(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
                 arr_store_si128(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
                 arr_store_si128(result->stats->tables->similar_table, vHS, i, segLen, j, s2Len);
@@ -386,21 +494,21 @@ parasail_result_t* PNAME(
                 arr_store_si128(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
 #endif
                 /* Update vF value. */
-                vEF_opn = _mm_sub_epi16(vH, vGapO);
-                vF_ext = _mm_sub_epi16(vF, vGapE);
+                vEF_opn = _mm_sub_epi32(vH, vGapO);
+                vF_ext = _mm_sub_epi32(vF, vGapE);
                 if (! _mm_movemask_epi8(
                             _mm_or_si128(
-                                _mm_cmpgt_epi16(vF_ext, vEF_opn),
-                                _mm_cmpeq_epi16(vF_ext, vEF_opn))))
+                                _mm_cmpgt_epi32(vF_ext, vEF_opn),
+                                _mm_cmpeq_epi32(vF_ext, vEF_opn))))
                     goto end;
-                /*vF = _mm_max_epi16(vEF_opn, vF_ext);*/
+                /*vF = _mm_max_epi32(vEF_opn, vF_ext);*/
                 vF = vF_ext;
-                cond = _mm_cmpgt_epi16(vEF_opn, vF_ext);
+                cond = _mm_cmpgt_epi32(vEF_opn, vF_ext);
                 vFM = _mm_blendv_epi8(vFM, vHM, cond);
                 vFS = _mm_blendv_epi8(vFS, vHS, cond);
                 vFL = _mm_blendv_epi8(
-                        _mm_add_epi16(vFL, vOne),
-                        _mm_add_epi16(vHL, vOne),
+                        _mm_add_epi32(vFL, vOne),
+                        _mm_add_epi32(vHL, vOne),
                         cond);
                 vHp = _mm_load_si128(pvHLoad + i);
             }
@@ -416,7 +524,7 @@ end:
             vHM = _mm_load_si128(pvHMStore + offset);
             vHS = _mm_load_si128(pvHSStore + offset);
             vHL = _mm_load_si128(pvHLStore + offset);
-            cond_max = _mm_cmpgt_epi16(vH, vMaxH);
+            cond_max = _mm_cmpgt_epi32(vH, vMaxH);
             vMaxH = _mm_blendv_epi8(vMaxH, vH, cond_max);
             vMaxHM = _mm_blendv_epi8(vMaxHM, vHM, cond_max);
             vMaxHS = _mm_blendv_epi8(vMaxHS, vHS, cond_max);
@@ -427,15 +535,15 @@ end:
         }
 #ifdef PARASAIL_ROWCOL
         for (k=0; k<position; ++k) {
-            vH = _mm_slli_si128(vH, 2);
-            vHM = _mm_slli_si128(vHM, 2);
-            vHS = _mm_slli_si128(vHS, 2);
-            vHL = _mm_slli_si128(vHL, 2);
+            vH = _mm_slli_si128(vH, 4);
+            vHM = _mm_slli_si128(vHM, 4);
+            vHS = _mm_slli_si128(vHS, 4);
+            vHL = _mm_slli_si128(vHL, 4);
         }
-        result->stats->rowcols->score_row[j] = (int16_t) _mm_extract_epi16 (vH, 7);
-        result->stats->rowcols->matches_row[j] = (int16_t) _mm_extract_epi16 (vHM, 7);
-        result->stats->rowcols->similar_row[j] = (int16_t) _mm_extract_epi16 (vHS, 7);
-        result->stats->rowcols->length_row[j] = (int16_t) _mm_extract_epi16 (vHL, 7);
+        result->stats->rowcols->score_row[j] = (int32_t) _mm_extract_epi32 (vH, 3);
+        result->stats->rowcols->matches_row[j] = (int32_t) _mm_extract_epi32 (vHM, 3);
+        result->stats->rowcols->similar_row[j] = (int32_t) _mm_extract_epi32 (vHS, 3);
+        result->stats->rowcols->length_row[j] = (int32_t) _mm_extract_epi32 (vHL, 3);
 #endif
     }
 
@@ -456,26 +564,26 @@ end:
     if (s2_end)
     {
         for (k=0; k<position; ++k) {
-            vMaxH = _mm_slli_si128(vMaxH, 2);
-            vMaxHM = _mm_slli_si128(vMaxHM, 2);
-            vMaxHS = _mm_slli_si128(vMaxHS, 2);
-            vMaxHL = _mm_slli_si128(vMaxHL, 2);
+            vMaxH = _mm_slli_si128(vMaxH, 4);
+            vMaxHM = _mm_slli_si128(vMaxHM, 4);
+            vMaxHS = _mm_slli_si128(vMaxHS, 4);
+            vMaxHL = _mm_slli_si128(vMaxHL, 4);
         }
         end_query = s1Len-1;
-        score = (int16_t) _mm_extract_epi16(vMaxH, 7);
-        matches = (int16_t) _mm_extract_epi16(vMaxHM, 7);
-        similar = (int16_t) _mm_extract_epi16(vMaxHS, 7);
-        length = (int16_t) _mm_extract_epi16(vMaxHL, 7);
+        score = (int32_t) _mm_extract_epi32(vMaxH, 3);
+        matches = (int32_t) _mm_extract_epi32(vMaxHM, 3);
+        similar = (int32_t) _mm_extract_epi32(vMaxHS, 3);
+        length = (int32_t) _mm_extract_epi32(vMaxHL, 3);
     }
 
     /* max of last column */
     if (s1_end)
     {
         /* Trace the alignment ending position on read. */
-        int16_t *t = (int16_t*)pvHStore;
-        int16_t *m = (int16_t*)pvHMStore;
-        int16_t *s = (int16_t*)pvHSStore;
-        int16_t *l = (int16_t*)pvHLStore;
+        int32_t *t = (int32_t*)pvHStore;
+        int32_t *m = (int32_t*)pvHMStore;
+        int32_t *s = (int32_t*)pvHSStore;
+        int32_t *l = (int32_t*)pvHLStore;
         int32_t column_len = segLen * segWidth;
         for (i = 0; i<column_len; ++i, ++t, ++m, ++s, ++l) {
             int32_t temp = i / segWidth + i % segWidth * segLen;
@@ -505,22 +613,22 @@ end:
         __m128i vHS = _mm_load_si128(pvHSStore + offset);
         __m128i vHL = _mm_load_si128(pvHLStore + offset);
         for (k=0; k<position; ++k) {
-            vH = _mm_slli_si128(vH, 2);
-            vHM = _mm_slli_si128(vHM, 2);
-            vHS = _mm_slli_si128(vHS, 2);
-            vHL = _mm_slli_si128(vHL, 2);
+            vH = _mm_slli_si128(vH, 4);
+            vHM = _mm_slli_si128(vHM, 4);
+            vHS = _mm_slli_si128(vHS, 4);
+            vHL = _mm_slli_si128(vHL, 4);
         }
         end_ref = s2Len - 1;
         end_query = s1Len - 1;
-        score = (int16_t) _mm_extract_epi16 (vH, 7);
-        matches = (int16_t) _mm_extract_epi16 (vHM, 7);
-        similar = (int16_t) _mm_extract_epi16 (vHS, 7);
-        length = (int16_t) _mm_extract_epi16 (vHL, 7);
+        score = (int32_t) _mm_extract_epi32 (vH, 3);
+        matches = (int32_t) _mm_extract_epi32 (vHM, 3);
+        similar = (int32_t) _mm_extract_epi32 (vHS, 3);
+        length = (int32_t) _mm_extract_epi32 (vHL, 3);
     }
 
     if (_mm_movemask_epi8(_mm_or_si128(
-            _mm_cmplt_epi16(vSaturationCheckMin, vNegLimit),
-            _mm_cmpgt_epi16(vSaturationCheckMax, vPosLimit)))) {
+            _mm_cmplt_epi32(vSaturationCheckMin, vNegLimit),
+            _mm_cmpgt_epi32(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         matches = 0;
@@ -536,19 +644,6 @@ end:
     result->stats->matches = matches;
     result->stats->similar = similar;
     result->stats->length = length;
-    result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_STRIPED
-        | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_8;
-    result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
-    result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
-    result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
-    result->flag |= s2_end ? PARASAIL_FLAG_SG_S2_END : 0;
-#ifdef PARASAIL_TABLE
-    result->flag |= PARASAIL_FLAG_TABLE;
-#endif
-#ifdef PARASAIL_ROWCOL
-    result->flag |= PARASAIL_FLAG_ROWCOL;
-#endif
 
     parasail_free(boundary);
     parasail_free(pvEL);
