@@ -12,18 +12,17 @@
 
 
 
-#define SG_TRACE
-#define SG_SUFFIX _striped_neon_128_16
-#define SG_SUFFIX_PROF _striped_profile_neon_128_16
-#include "sg_helper.h"
-
 #include "parasail.h"
 #include "parasail/memory.h"
 #include "parasail/internal_neon.h"
 
+#define SG_TRACE
+#define SG_SUFFIX _striped_neon_128_64
+#define SG_SUFFIX_PROF _striped_profile_neon_128_64
+#include "sg_helper.h"
+
 #define SWAP(A,B) { simde__m128i* tmp = A; A = B; B = tmp; }
 
-#define NEG_INF (INT16_MIN/(int16_t)(2))
 
 
 static inline void arr_store(
@@ -45,8 +44,8 @@ static inline simde__m128i arr_load(
     return simde_mm_load_si128(array + (1LL*d*seglen+t));
 }
 
-#define FNAME parasail_sg_flags_trace_striped_neon_128_16
-#define PNAME parasail_sg_flags_trace_striped_profile_neon_128_16
+#define FNAME parasail_sg_flags_trace_striped_neon_128_64
+#define PNAME parasail_sg_flags_trace_striped_profile_neon_128_64
 
 parasail_result_t* FNAME(
         const char * const restrict s1, const int s1Len,
@@ -54,9 +53,28 @@ parasail_result_t* FNAME(
         const int open, const int gap, const parasail_matrix_t *matrix,
         int s1_beg, int s1_end, int s2_beg, int s2_end)
 {
-    parasail_profile_t *profile = parasail_profile_create_neon_128_16(s1, s1Len, matrix);
-    parasail_result_t *result = PNAME(profile, s2, s2Len, open, gap, s1_beg, s1_end, s2_beg, s2_end);
+    /* declare local variables */
+    parasail_profile_t *profile = NULL;
+    parasail_result_t *result = NULL;
+
+    /* validate inputs */
+    PARASAIL_CHECK_NULL(s2);
+    PARASAIL_CHECK_GT0(s2Len);
+    PARASAIL_CHECK_GE0(open);
+    PARASAIL_CHECK_GE0(gap);
+    PARASAIL_CHECK_NULL(matrix);
+    if (matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) {
+        PARASAIL_CHECK_NULL(s1);
+        PARASAIL_CHECK_GT0(s1Len);
+    }
+
+    /* initialize local variables */
+    profile = parasail_profile_create_neon_128_64(s1, s1Len, matrix);
+    if (!profile) return NULL;
+    result = PNAME(profile, s2, s2Len, open, gap, s1_beg, s1_end, s2_beg, s2_end);
+
     parasail_profile_free(profile);
+
     return result;
 }
 
@@ -66,43 +84,123 @@ parasail_result_t* PNAME(
         const int open, const int gap,
         int s1_beg, int s1_end, int s2_beg, int s2_end)
 {
+    /* declare local variables */
     int32_t i = 0;
     int32_t j = 0;
     int32_t k = 0;
-    const int s1Len = profile->s1Len;
-    int32_t end_query = s1Len-1;
-    int32_t end_ref = s2Len-1;
-    const parasail_matrix_t *matrix = profile->matrix;
-    const int32_t segWidth = 8; /* number of values in vector unit */
-    const int32_t segLen = (s1Len + segWidth - 1) / segWidth;
-    const int32_t offset = (s1Len - 1) % segLen;
-    const int32_t position = (segWidth - 1) - (s1Len - 1) / segLen;
-    simde__m128i* const restrict vProfile = (simde__m128i*)profile->profile16.score;
-    simde__m128i* restrict pvHStore = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* restrict pvHLoad = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* const restrict pvE = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* restrict pvEaStore = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* restrict pvEaLoad = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* const restrict pvHT = parasail_memalign_simde__m128i(16, segLen);
-    int16_t* const restrict boundary = parasail_memalign_int16_t(16, s2Len+1);
-    simde__m128i vGapO = simde_mm_set1_epi16(open);
-    simde__m128i vGapE = simde_mm_set1_epi16(gap);
-    simde__m128i vNegInf = simde_mm_set1_epi16(NEG_INF);
-    int16_t score = NEG_INF;
-    simde__m128i vMaxH = vNegInf;
-    simde__m128i vPosMask = simde_mm_cmpeq_epi16(simde_mm_set1_epi16(position),
-            simde_mm_set_epi16(0,1,2,3,4,5,6,7));
-    
-    parasail_result_t *result = parasail_result_new_trace(segLen, s2Len, 16, sizeof(simde__m128i));
-    simde__m128i vTIns  = simde_mm_set1_epi16(PARASAIL_INS);
-    simde__m128i vTDel  = simde_mm_set1_epi16(PARASAIL_DEL);
-    simde__m128i vTDiag = simde_mm_set1_epi16(PARASAIL_DIAG);
-    simde__m128i vTDiagE = simde_mm_set1_epi16(PARASAIL_DIAG_E);
-    simde__m128i vTInsE = simde_mm_set1_epi16(PARASAIL_INS_E);
-    simde__m128i vTDiagF = simde_mm_set1_epi16(PARASAIL_DIAG_F);
-    simde__m128i vTDelF = simde_mm_set1_epi16(PARASAIL_DEL_F);
-    simde__m128i vTMask = simde_mm_set1_epi16(PARASAIL_ZERO_MASK);
-    simde__m128i vFTMask = simde_mm_set1_epi16(PARASAIL_F_MASK);
+    int32_t s1Len = 0;
+    int32_t end_query = 0;
+    int32_t end_ref = 0;
+    const parasail_matrix_t *matrix = NULL;
+    int32_t segWidth = 0;
+    int32_t segLen = 0;
+    int32_t offset = 0;
+    int32_t position = 0;
+    simde__m128i* restrict vProfile = NULL;
+    simde__m128i* restrict pvHStore = NULL;
+    simde__m128i* restrict pvHLoad = NULL;
+    simde__m128i* restrict pvE = NULL;
+    simde__m128i* restrict pvEaStore = NULL;
+    simde__m128i* restrict pvEaLoad = NULL;
+    simde__m128i* restrict pvHT = NULL;
+    int64_t* restrict boundary = NULL;
+    simde__m128i vGapO;
+    simde__m128i vGapE;
+    int64_t NEG_LIMIT = 0;
+    int64_t POS_LIMIT = 0;
+    int64_t score = 0;
+    simde__m128i vNegLimit;
+    simde__m128i vPosLimit;
+    simde__m128i vSaturationCheckMin;
+    simde__m128i vSaturationCheckMax;
+    simde__m128i vMaxH;
+    simde__m128i vPosMask;
+    parasail_result_t *result = NULL;
+    simde__m128i vTIns;
+    simde__m128i vTDel;
+    simde__m128i vTDiag;
+    simde__m128i vTDiagE;
+    simde__m128i vTInsE;
+    simde__m128i vTDiagF;
+    simde__m128i vTDelF;
+    simde__m128i vTMask;
+    simde__m128i vFTMask;
+
+    /* validate inputs */
+    PARASAIL_CHECK_NULL(profile);
+    PARASAIL_CHECK_NULL(profile->profile64.score);
+    PARASAIL_CHECK_NULL(profile->matrix);
+    PARASAIL_CHECK_GT0(profile->s1Len);
+    PARASAIL_CHECK_NULL(s2);
+    PARASAIL_CHECK_GT0(s2Len);
+    PARASAIL_CHECK_GE0(open);
+    PARASAIL_CHECK_GE0(gap);
+
+    /* initialize stack variables */
+    i = 0;
+    j = 0;
+    k = 0;
+    s1Len = profile->s1Len;
+    end_query = s1Len-1;
+    end_ref = s2Len-1;
+    matrix = profile->matrix;
+    segWidth = 2; /* number of values in vector unit */
+    segLen = (s1Len + segWidth - 1) / segWidth;
+    offset = (s1Len - 1) % segLen;
+    position = (segWidth - 1) - (s1Len - 1) / segLen;
+    vProfile = (simde__m128i*)profile->profile64.score;
+    vGapO = simde_mm_set1_epi64x(open);
+    vGapE = simde_mm_set1_epi64x(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
+    POS_LIMIT = INT64_MAX - matrix->max - 1;
+    score = NEG_LIMIT;
+    vNegLimit = simde_mm_set1_epi64x(NEG_LIMIT);
+    vPosLimit = simde_mm_set1_epi64x(POS_LIMIT);
+    vSaturationCheckMin = vPosLimit;
+    vSaturationCheckMax = vNegLimit;
+    vMaxH = vNegLimit;
+    vPosMask = simde_mm_cmpeq_epi64(simde_mm_set1_epi64x(position),
+            simde_mm_set_epi64x(0,1));
+    vTIns  = simde_mm_set1_epi64x(PARASAIL_INS);
+    vTDel  = simde_mm_set1_epi64x(PARASAIL_DEL);
+    vTDiag = simde_mm_set1_epi64x(PARASAIL_DIAG);
+    vTDiagE= simde_mm_set1_epi64x(PARASAIL_DIAG_E);
+    vTInsE = simde_mm_set1_epi64x(PARASAIL_INS_E);
+    vTDiagF= simde_mm_set1_epi64x(PARASAIL_DIAG_F);
+    vTDelF = simde_mm_set1_epi64x(PARASAIL_DEL_F);
+    vTMask = simde_mm_set1_epi64x(PARASAIL_ZERO_MASK);
+    vFTMask= simde_mm_set1_epi64x(PARASAIL_F_MASK);
+
+    /* initialize result */
+    result = parasail_result_new_trace(segLen, s2Len, 16, sizeof(simde__m128i));
+    if (!result) return NULL;
+
+    /* set known flags */
+    result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_STRIPED
+        | PARASAIL_FLAG_TRACE
+        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_2;
+    result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
+    result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
+    result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
+    result->flag |= s2_end ? PARASAIL_FLAG_SG_S2_END : 0;
+
+    /* initialize heap variables */
+    pvHStore = parasail_memalign_simde__m128i(16, segLen);
+    pvHLoad  = parasail_memalign_simde__m128i(16, segLen);
+    pvE      = parasail_memalign_simde__m128i(16, segLen);
+    pvEaStore= parasail_memalign_simde__m128i(16, segLen);
+    pvEaLoad = parasail_memalign_simde__m128i(16, segLen);
+    pvHT     = parasail_memalign_simde__m128i(16, segLen);
+    boundary = parasail_memalign_int64_t(16, s2Len+1);
+
+    /* validate heap variables */
+    if (!pvHStore) return NULL;
+    if (!pvHLoad) return NULL;
+    if (!pvE) return NULL;
+    if (!pvEaStore) return NULL;
+    if (!pvEaLoad) return NULL;
+    if (!pvHT) return NULL;
+    if (!boundary) return NULL;
 
     /* initialize H and E */
     {
@@ -113,9 +211,9 @@ parasail_result_t* PNAME(
             simde__m128i e;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = s1_beg ? 0 : (-open-gap*(segNum*segLen+i));
-                h.i16[segNum] = tmp < INT16_MIN ? INT16_MIN : tmp;
+                h.i64[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
                 tmp = tmp - open;
-                e.i16[segNum] = tmp < INT16_MIN ? INT16_MIN : tmp;
+                e.i64[segNum] = tmp < INT64_MIN ? INT64_MIN : tmp;
             }
             simde_mm_store_si128(&pvHStore[index], h);
             simde_mm_store_si128(&pvE[index], e);
@@ -129,7 +227,7 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = s2_beg ? 0 : (-open-gap*(i-1));
-            boundary[i] = tmp < INT16_MIN ? INT16_MIN : tmp;
+            boundary[i] = tmp < INT64_MIN ? INT64_MIN : tmp;
         }
     }
 
@@ -152,14 +250,14 @@ parasail_result_t* PNAME(
 
         /* Initialize F value to -inf.  Any errors to vH values will be
          * corrected in the Lazy_F loop. */
-        vF = vNegInf;
+        vF = vNegLimit;
 
-        /* load final segment of pvHStore and shift left by 2 bytes */
+        /* load final segment of pvHStore and shift left by 8 bytes */
         vH = simde_mm_load_si128(&pvHStore[segLen - 1]);
-        vH = simde_mm_slli_si128(vH, 2);
+        vH = simde_mm_slli_si128(vH, 8);
 
         /* insert upper boundary condition */
-        vH = simde_mm_insert_epi16(vH, boundary[j], 0);
+        vH = simde_mm_insert_epi64(vH, boundary[j], 0);
 
         /* Correct part of the vProfile */
         vP = vProfile + matrix->mapper[(unsigned char)s2[j]] * segLen;
@@ -173,17 +271,20 @@ parasail_result_t* PNAME(
             vE = simde_mm_load_si128(pvE + i);
 
             /* Get max from vH, vE and vF. */
-            vH_dag = simde_mm_add_epi16(vH, simde_mm_load_si128(vP + i));
-            vH = simde_mm_max_epi16(vH_dag, vE);
-            vH = simde_mm_max_epi16(vH, vF);
+            vH_dag = simde_mm_add_epi64(vH, simde_mm_load_si128(vP + i));
+            vH = simde_mm_max_epi64(vH_dag, vE);
+            vH = simde_mm_max_epi64(vH, vF);
             /* Save vH values. */
             simde_mm_store_si128(pvHStore + i, vH);
-            
+            vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vH);
+            vSaturationCheckMin = simde_mm_min_epi64(vSaturationCheckMin, vH);
+            vSaturationCheckMin = simde_mm_min_epi64(vSaturationCheckMin, vE);
+            vSaturationCheckMin = simde_mm_min_epi64(vSaturationCheckMin, vF);
 
             {
                 simde__m128i vTAll = arr_load(result->trace->trace_table, i, segLen, j);
-                simde__m128i case1 = simde_mm_cmpeq_epi16(vH, vH_dag);
-                simde__m128i case2 = simde_mm_cmpeq_epi16(vH, vF);
+                simde__m128i case1 = simde_mm_cmpeq_epi64(vH, vH_dag);
+                simde__m128i case2 = simde_mm_cmpeq_epi64(vH, vF);
                 simde__m128i vT = simde_mm_blendv_epi8(
                         simde_mm_blendv_epi8(vTIns, vTDel, case2),
                         vTDiag, case1);
@@ -192,30 +293,30 @@ parasail_result_t* PNAME(
                 arr_store(result->trace->trace_table, vT, i, segLen, j);
             }
 
-            vEF_opn = simde_mm_sub_epi16(vH, vGapO);
+            vEF_opn = simde_mm_sub_epi64(vH, vGapO);
 
             /* Update vE value. */
-            vE_ext = simde_mm_sub_epi16(vE, vGapE);
-            vE = simde_mm_max_epi16(vEF_opn, vE_ext);
+            vE_ext = simde_mm_sub_epi64(vE, vGapE);
+            vE = simde_mm_max_epi64(vEF_opn, vE_ext);
             simde_mm_store_si128(pvE + i, vE);
             {
                 simde__m128i vEa = simde_mm_load_si128(pvEaLoad + i);
-                simde__m128i vEa_ext = simde_mm_sub_epi16(vEa, vGapE);
-                vEa = simde_mm_max_epi16(vEF_opn, vEa_ext);
+                simde__m128i vEa_ext = simde_mm_sub_epi64(vEa, vGapE);
+                vEa = simde_mm_max_epi64(vEF_opn, vEa_ext);
                 simde_mm_store_si128(pvEaStore + i, vEa);
                 if (j+1<s2Len) {
-                    simde__m128i cond = simde_mm_cmpgt_epi16(vEF_opn, vEa_ext);
+                    simde__m128i cond = simde_mm_cmpgt_epi64(vEF_opn, vEa_ext);
                     simde__m128i vT = simde_mm_blendv_epi8(vTInsE, vTDiagE, cond);
                     arr_store(result->trace->trace_table, vT, i, segLen, j+1);
                 }
             }
 
             /* Update vF value. */
-            vF_ext = simde_mm_sub_epi16(vF, vGapE);
-            vF = simde_mm_max_epi16(vEF_opn, vF_ext);
+            vF_ext = simde_mm_sub_epi64(vF, vGapE);
+            vF = simde_mm_max_epi64(vEF_opn, vF_ext);
             if (i+1<segLen) {
                 simde__m128i vTAll = arr_load(result->trace->trace_table, i+1, segLen, j);
-                simde__m128i cond = simde_mm_cmpgt_epi16(vEF_opn, vF_ext);
+                simde__m128i cond = simde_mm_cmpgt_epi64(vEF_opn, vF_ext);
                 simde__m128i vT = simde_mm_blendv_epi8(vTDelF, vTDiagF, cond);
                 vT = simde_mm_or_si128(vT, vTAll);
                 arr_store(result->trace->trace_table, vT, i+1, segLen, j);
@@ -231,34 +332,35 @@ parasail_result_t* PNAME(
         vFa = vF;
         for (k=0; k<segWidth; ++k) {
             int64_t tmp = s2_beg ? -open : (boundary[j+1]-open);
-            int16_t tmp2 = tmp < INT16_MIN ? INT16_MIN : tmp;
+            int64_t tmp2 = tmp < INT64_MIN ? INT64_MIN : tmp;
             simde__m128i vHp = simde_mm_load_si128(&pvHLoad[segLen - 1]);
-            vHp = simde_mm_slli_si128(vHp, 2);
-            vHp = simde_mm_insert_epi16(vHp, boundary[j], 0);
-            vEF_opn = simde_mm_slli_si128(vEF_opn, 2);
-            vEF_opn = simde_mm_insert_epi16(vEF_opn, tmp2, 0);
-            vF_ext = simde_mm_slli_si128(vF_ext, 2);
-            vF_ext = simde_mm_insert_epi16(vF_ext, NEG_INF, 0);
-            vF = simde_mm_slli_si128(vF, 2);
-            vF = simde_mm_insert_epi16(vF, tmp2, 0);
-            vFa_ext = simde_mm_slli_si128(vFa_ext, 2);
-            vFa_ext = simde_mm_insert_epi16(vFa_ext, NEG_INF, 0);
-            vFa = simde_mm_slli_si128(vFa, 2);
-            vFa = simde_mm_insert_epi16(vFa, tmp2, 0);
+            vHp = simde_mm_slli_si128(vHp, 8);
+            vHp = simde_mm_insert_epi64(vHp, boundary[j], 0);
+            vEF_opn = simde_mm_slli_si128(vEF_opn, 8);
+            vEF_opn = simde_mm_insert_epi64(vEF_opn, tmp2, 0);
+            vF_ext = simde_mm_slli_si128(vF_ext, 8);
+            vF_ext = simde_mm_insert_epi64(vF_ext, NEG_LIMIT, 0);
+            vF = simde_mm_slli_si128(vF, 8);
+            vF = simde_mm_insert_epi64(vF, tmp2, 0);
+            vFa_ext = simde_mm_slli_si128(vFa_ext, 8);
+            vFa_ext = simde_mm_insert_epi64(vFa_ext, NEG_LIMIT, 0);
+            vFa = simde_mm_slli_si128(vFa, 8);
+            vFa = simde_mm_insert_epi64(vFa, tmp2, 0);
             for (i=0; i<segLen; ++i) {
                 vH = simde_mm_load_si128(pvHStore + i);
-                vH = simde_mm_max_epi16(vH,vF);
+                vH = simde_mm_max_epi64(vH,vF);
                 simde_mm_store_si128(pvHStore + i, vH);
-                
+                vSaturationCheckMin = simde_mm_min_epi64(vSaturationCheckMin, vH);
+                vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vH);
                 {
                     simde__m128i vTAll;
                     simde__m128i vT;
                     simde__m128i case1;
                     simde__m128i case2;
                     simde__m128i cond;
-                    vHp = simde_mm_add_epi16(vHp, simde_mm_load_si128(vP + i));
-                    case1 = simde_mm_cmpeq_epi16(vH, vHp);
-                    case2 = simde_mm_cmpeq_epi16(vH, vF);
+                    vHp = simde_mm_add_epi64(vHp, simde_mm_load_si128(vP + i));
+                    case1 = simde_mm_cmpeq_epi64(vH, vHp);
+                    case2 = simde_mm_cmpeq_epi64(vH, vF);
                     cond = simde_mm_andnot_si128(case1,case2);
                     vTAll = arr_load(result->trace->trace_table, i, segLen, j);
                     vT = simde_mm_load_si128(pvHT + i);
@@ -271,34 +373,34 @@ parasail_result_t* PNAME(
                 /* Update vF value. */
                 {
                     simde__m128i vTAll = arr_load(result->trace->trace_table, i, segLen, j);
-                    simde__m128i cond = simde_mm_cmpgt_epi16(vEF_opn, vFa_ext);
+                    simde__m128i cond = simde_mm_cmpgt_epi64(vEF_opn, vFa_ext);
                     simde__m128i vT = simde_mm_blendv_epi8(vTDelF, vTDiagF, cond);
                     vTAll = simde_mm_and_si128(vTAll, vFTMask);
                     vTAll = simde_mm_or_si128(vTAll, vT);
                     arr_store(result->trace->trace_table, vTAll, i, segLen, j);
                 }
-                vEF_opn = simde_mm_sub_epi16(vH, vGapO);
-                vF_ext = simde_mm_sub_epi16(vF, vGapE);
+                vEF_opn = simde_mm_sub_epi64(vH, vGapO);
+                vF_ext = simde_mm_sub_epi64(vF, vGapE);
                 {
                     simde__m128i vEa = simde_mm_load_si128(pvEaLoad + i);
-                    simde__m128i vEa_ext = simde_mm_sub_epi16(vEa, vGapE);
-                    vEa = simde_mm_max_epi16(vEF_opn, vEa_ext);
+                    simde__m128i vEa_ext = simde_mm_sub_epi64(vEa, vGapE);
+                    vEa = simde_mm_max_epi64(vEF_opn, vEa_ext);
                     simde_mm_store_si128(pvEaStore + i, vEa);
                     if (j+1<s2Len) {
-                        simde__m128i cond = simde_mm_cmpgt_epi16(vEF_opn, vEa_ext);
+                        simde__m128i cond = simde_mm_cmpgt_epi64(vEF_opn, vEa_ext);
                         simde__m128i vT = simde_mm_blendv_epi8(vTInsE, vTDiagE, cond);
                         arr_store(result->trace->trace_table, vT, i, segLen, j+1);
                     }
                 }
                 if (! simde_mm_movemask_epi8(
                             simde_mm_or_si128(
-                                simde_mm_cmpgt_epi16(vF_ext, vEF_opn),
-                                simde_mm_cmpeq_epi16(vF_ext, vEF_opn))))
+                                simde_mm_cmpgt_epi64(vF_ext, vEF_opn),
+                                simde_mm_cmpeq_epi64(vF_ext, vEF_opn))))
                     goto end;
-                /*vF = simde_mm_max_epi16(vEF_opn, vF_ext);*/
+                /*vF = simde_mm_max_epi64(vEF_opn, vF_ext);*/
                 vF = vF_ext;
-                vFa_ext = simde_mm_sub_epi16(vFa, vGapE);
-                vFa = simde_mm_max_epi16(vEF_opn, vFa_ext);
+                vFa_ext = simde_mm_sub_epi64(vFa, vGapE);
+                vFa = simde_mm_max_epi64(vEF_opn, vFa_ext);
                 vHp = simde_mm_load_si128(pvHLoad + i);
             }
         }
@@ -307,8 +409,8 @@ end:
             /* extract vector containing last value from the column */
             simde__m128i vCompare;
             vH = simde_mm_load_si128(pvHStore + offset);
-            vCompare = simde_mm_and_si128(vPosMask, simde_mm_cmpgt_epi16(vH, vMaxH));
-            vMaxH = simde_mm_max_epi16(vH, vMaxH);
+            vCompare = simde_mm_and_si128(vPosMask, simde_mm_cmpgt_epi64(vH, vMaxH));
+            vMaxH = simde_mm_max_epi64(vH, vMaxH);
             if (simde_mm_movemask_epi8(vCompare)) {
                 end_ref = j;
             }
@@ -319,9 +421,9 @@ end:
     if (s2_end)
     {
         for (k=0; k<position; ++k) {
-            vMaxH = simde_mm_slli_si128(vMaxH, 2);
+            vMaxH = simde_mm_slli_si128(vMaxH, 8);
         }
-        score = (int16_t) simde_mm_extract_epi16(vMaxH, 7);
+        score = (int64_t) simde_mm_extract_epi64(vMaxH, 1);
         end_query = s1Len-1;
     }
 
@@ -329,7 +431,7 @@ end:
     if (s1_end)
     {
         /* Trace the alignment ending position on read. */
-        int16_t *t = (int16_t*)pvHStore;
+        int64_t *t = (int64_t*)pvHStore;
         int32_t column_len = segLen * segWidth;
         for (i = 0; i<column_len; ++i, ++t) {
             int32_t temp = i / segWidth + i % segWidth * segLen;
@@ -350,26 +452,26 @@ end:
         {
             simde__m128i vH = simde_mm_load_si128(pvHStore + offset);
             for (k=0; k<position; ++k) {
-                vH = simde_mm_slli_si128(vH, 2);
+                vH = simde_mm_slli_si128(vH, 8);
             }
-            score = (int16_t) simde_mm_extract_epi16 (vH, 7);
+            score = (int64_t) simde_mm_extract_epi64 (vH, 1);
             end_ref = s2Len - 1;
             end_query = s1Len - 1;
         }
     }
 
-    
+    if (simde_mm_movemask_epi8(simde_mm_or_si128(
+            simde_mm_cmplt_epi64(vSaturationCheckMin, vNegLimit),
+            simde_mm_cmpgt_epi64(vSaturationCheckMax, vPosLimit)))) {
+        result->flag |= PARASAIL_FLAG_SATURATED;
+        score = 0;
+        end_query = 0;
+        end_ref = 0;
+    }
 
     result->score = score;
     result->end_query = end_query;
     result->end_ref = end_ref;
-    result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_STRIPED
-        | PARASAIL_FLAG_TRACE
-        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_8;
-    result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
-    result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
-    result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
-    result->flag |= s2_end ? PARASAIL_FLAG_SG_S2_END : 0;
 
     parasail_free(boundary);
     parasail_free(pvHT);
