@@ -164,76 +164,179 @@ static inline void arr_store_rowcol(
 #endif
 
 parasail_result_t* FNAME(
-        const char * const restrict _s1, const int s1Len,
+        const char * const restrict _s1, const int _s1Len,
         const char * const restrict _s2, const int s2Len,
         const int open, const int gap, const parasail_matrix_t *matrix)
 {
-    const int32_t N = 4; /* number of values in vector */
-    const int32_t PAD = N-1;
-    const int32_t PAD2 = PAD*2;
-    const int32_t s1Len_PAD = s1Len+PAD;
-    const int32_t s2Len_PAD = s2Len+PAD;
-    int64_t * const restrict s1      = parasail_memalign_int64_t(32, s1Len+PAD);
-    int64_t * const restrict s2B     = parasail_memalign_int64_t(32, s2Len+PAD2);
-    int64_t * const restrict _H_pr = parasail_memalign_int64_t(32, s2Len+PAD2);
-    int64_t * const restrict _HM_pr = parasail_memalign_int64_t(32, s2Len+PAD2);
-    int64_t * const restrict _HS_pr = parasail_memalign_int64_t(32, s2Len+PAD2);
-    int64_t * const restrict _HL_pr = parasail_memalign_int64_t(32, s2Len+PAD2);
-    int64_t * const restrict _F_pr = parasail_memalign_int64_t(32, s2Len+PAD2);
-    int64_t * const restrict _FM_pr = parasail_memalign_int64_t(32, s2Len+PAD2);
-    int64_t * const restrict _FS_pr = parasail_memalign_int64_t(32, s2Len+PAD2);
-    int64_t * const restrict _FL_pr = parasail_memalign_int64_t(32, s2Len+PAD2);
-    int64_t * const restrict s2 = s2B+PAD; /* will allow later for negative indices */
-    int64_t * const restrict H_pr = _H_pr+PAD;
-    int64_t * const restrict HM_pr = _HM_pr+PAD;
-    int64_t * const restrict HS_pr = _HS_pr+PAD;
-    int64_t * const restrict HL_pr = _HL_pr+PAD;
-    int64_t * const restrict F_pr = _F_pr+PAD;
-    int64_t * const restrict FM_pr = _FM_pr+PAD;
-    int64_t * const restrict FS_pr = _FS_pr+PAD;
-    int64_t * const restrict FL_pr = _FL_pr+PAD;
-#ifdef PARASAIL_TABLE
-    parasail_result_t *result = parasail_result_new_table3(s1Len, s2Len);
-#else
-#ifdef PARASAIL_ROWCOL
-    parasail_result_t *result = parasail_result_new_rowcol3(s1Len, s2Len);
-#else
-    parasail_result_t *result = parasail_result_new_stats();
-#endif
-#endif
+    /* declare local variables */
+    int32_t N = 0;
+    int32_t PAD = 0;
+    int32_t PAD2 = 0;
+    int32_t s1Len = 0;
+    int32_t s1Len_PAD = 0;
+    int32_t s2Len_PAD = 0;
+    int64_t * restrict s1 = NULL;
+    int64_t * restrict s2B = NULL;
+    int64_t * restrict _H_pr = NULL;
+    int64_t * restrict _HM_pr = NULL;
+    int64_t * restrict _HS_pr = NULL;
+    int64_t * restrict _HL_pr = NULL;
+    int64_t * restrict _F_pr = NULL;
+    int64_t * restrict _FM_pr = NULL;
+    int64_t * restrict _FS_pr = NULL;
+    int64_t * restrict _FL_pr = NULL;
+    int64_t * restrict s2 = NULL;
+    int64_t * restrict H_pr = NULL;
+    int64_t * restrict HM_pr = NULL;
+    int64_t * restrict HS_pr = NULL;
+    int64_t * restrict HL_pr = NULL;
+    int64_t * restrict F_pr = NULL;
+    int64_t * restrict FM_pr = NULL;
+    int64_t * restrict FS_pr = NULL;
+    int64_t * restrict FL_pr = NULL;
+    parasail_result_t *result = NULL;
     int32_t i = 0;
     int32_t j = 0;
     int32_t end_query = 0;
     int32_t end_ref = 0;
-    const int64_t NEG_LIMIT = (-open < matrix->min ?
-        INT64_MIN + open : INT64_MIN - matrix->min) + 1;
-    const int64_t POS_LIMIT = INT64_MAX - matrix->max - 1;
-    int64_t score = NEG_LIMIT;
-    int64_t matches = NEG_LIMIT;
-    int64_t similar = NEG_LIMIT;
-    int64_t length = NEG_LIMIT;
-    __m256i vNegLimit = _mm256_set1_epi64x_rpl(NEG_LIMIT);
-    __m256i vPosLimit = _mm256_set1_epi64x_rpl(POS_LIMIT);
-    __m256i vSaturationCheckMin = vPosLimit;
-    __m256i vSaturationCheckMax = vNegLimit;
-    __m256i vNegInf = _mm256_set1_epi64x_rpl(NEG_LIMIT);
-    __m256i vNegInf0 = _mm256_srli_si256_rpl(vNegInf, 8); /* shift in a 0 */
-    __m256i vOpen = _mm256_set1_epi64x_rpl(open);
-    __m256i vGap  = _mm256_set1_epi64x_rpl(gap);
-    __m256i vZero = _mm256_set1_epi64x_rpl(0);
-    __m256i vOne = _mm256_set1_epi64x_rpl(1);
-    __m256i vN = _mm256_set1_epi64x_rpl(N);
-    __m256i vNegOne = _mm256_set1_epi64x_rpl(-1);
-    __m256i vI = _mm256_set_epi64x_rpl(0,1,2,3);
-    __m256i vJreset = _mm256_set_epi64x_rpl(0,-1,-2,-3);
-    __m256i vMaxH = vNegInf;
-    __m256i vMaxM = vNegInf;
-    __m256i vMaxS = vNegInf;
-    __m256i vMaxL = vNegInf;
-    __m256i vEndI = vNegInf;
-    __m256i vEndJ = vNegInf;
-    __m256i vILimit = _mm256_set1_epi64x_rpl(s1Len);
-    __m256i vJLimit = _mm256_set1_epi64x_rpl(s2Len);
+    int64_t NEG_LIMIT = 0;
+    int64_t POS_LIMIT = 0;
+    int64_t score = 0;
+    int64_t matches = 0;
+    int64_t similar = 0;
+    int64_t length = 0;
+    __m256i vNegLimit;
+    __m256i vPosLimit;
+    __m256i vSaturationCheckMin;
+    __m256i vSaturationCheckMax;
+    __m256i vNegInf;
+    __m256i vNegInf0;
+    __m256i vOpen;
+    __m256i vGap;
+    __m256i vZero;
+    __m256i vOne;
+    __m256i vN;
+    __m256i vNegOne;
+    __m256i vI;
+    __m256i vJreset;
+    __m256i vMaxH;
+    __m256i vMaxM;
+    __m256i vMaxS;
+    __m256i vMaxL;
+    __m256i vEndI;
+    __m256i vEndJ;
+    __m256i vILimit;
+    __m256i vJLimit;
+
+    /* validate inputs */
+    PARASAIL_CHECK_NULL(_s2);
+    PARASAIL_CHECK_GT0(s2Len);
+    PARASAIL_CHECK_GE0(open);
+    PARASAIL_CHECK_GE0(gap);
+    PARASAIL_CHECK_NULL(matrix);
+    if (matrix->type == PARASAIL_MATRIX_TYPE_PSSM) {
+        PARASAIL_CHECK_NULL_PSSM_STATS(_s1);
+    }
+    else {
+        PARASAIL_CHECK_NULL(_s1);
+        PARASAIL_CHECK_GT0(_s1Len);
+    }
+
+    /* initialize stack variables */
+    N = 4; /* number of values in vector */
+    PAD = N-1;
+    PAD2 = PAD*2;
+    s1Len = matrix->type == PARASAIL_MATRIX_TYPE_SQUARE ? _s1Len : matrix->length;
+    s1Len_PAD = s1Len+PAD;
+    s2Len_PAD = s2Len+PAD;
+    i = 0;
+    j = 0;
+    end_query = 0;
+    end_ref = 0;
+    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
+    POS_LIMIT = INT64_MAX - matrix->max - 1;
+    score = NEG_LIMIT;
+    matches = NEG_LIMIT;
+    similar = NEG_LIMIT;
+    length = NEG_LIMIT;
+    vNegLimit = _mm256_set1_epi64x_rpl(NEG_LIMIT);
+    vPosLimit = _mm256_set1_epi64x_rpl(POS_LIMIT);
+    vSaturationCheckMin = vPosLimit;
+    vSaturationCheckMax = vNegLimit;
+    vNegInf = _mm256_set1_epi64x_rpl(NEG_LIMIT);
+    vNegInf0 = _mm256_srli_si256_rpl(vNegInf, 8); /* shift in a 0 */
+    vOpen = _mm256_set1_epi64x_rpl(open);
+    vGap  = _mm256_set1_epi64x_rpl(gap);
+    vZero = _mm256_set1_epi64x_rpl(0);
+    vOne = _mm256_set1_epi64x_rpl(1);
+    vN = _mm256_set1_epi64x_rpl(N);
+    vNegOne = _mm256_set1_epi64x_rpl(-1);
+    vI = _mm256_set_epi64x_rpl(0,1,2,3);
+    vJreset = _mm256_set_epi64x_rpl(0,-1,-2,-3);
+    vMaxH = vNegInf;
+    vMaxM = vNegInf;
+    vMaxS = vNegInf;
+    vMaxL = vNegInf;
+    vEndI = vNegInf;
+    vEndJ = vNegInf;
+    vILimit = _mm256_set1_epi64x_rpl(s1Len);
+    vJLimit = _mm256_set1_epi64x_rpl(s2Len);
+
+    /* initialize result */
+#ifdef PARASAIL_TABLE
+    result = parasail_result_new_table3(s1Len, s2Len);
+#else
+#ifdef PARASAIL_ROWCOL
+    result = parasail_result_new_rowcol3(s1Len, s2Len);
+#else
+    result = parasail_result_new_stats();
+#endif
+#endif
+    if (!result) return NULL;
+
+    /* set known flags */
+    result->flag |= PARASAIL_FLAG_SW | PARASAIL_FLAG_DIAG
+        | PARASAIL_FLAG_STATS
+        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_4;
+#ifdef PARASAIL_TABLE
+    result->flag |= PARASAIL_FLAG_TABLE;
+#endif
+#ifdef PARASAIL_ROWCOL
+    result->flag |= PARASAIL_FLAG_ROWCOL;
+#endif
+
+    /* initialize heap variables */
+    s1     = parasail_memalign_int64_t(32, s1Len+PAD);
+    s2B    = parasail_memalign_int64_t(32, s2Len+PAD2);
+    _H_pr  = parasail_memalign_int64_t(32, s2Len+PAD2);
+    _HM_pr = parasail_memalign_int64_t(32, s2Len+PAD2);
+    _HS_pr = parasail_memalign_int64_t(32, s2Len+PAD2);
+    _HL_pr = parasail_memalign_int64_t(32, s2Len+PAD2);
+    _F_pr  = parasail_memalign_int64_t(32, s2Len+PAD2);
+    _FM_pr = parasail_memalign_int64_t(32, s2Len+PAD2);
+    _FS_pr = parasail_memalign_int64_t(32, s2Len+PAD2);
+    _FL_pr = parasail_memalign_int64_t(32, s2Len+PAD2);
+    s2 = s2B+PAD; /* will allow later for negative indices */
+    H_pr = _H_pr+PAD;
+    HM_pr = _HM_pr+PAD;
+    HS_pr = _HS_pr+PAD;
+    HL_pr = _HL_pr+PAD;
+    F_pr = _F_pr+PAD;
+    FM_pr = _FM_pr+PAD;
+    FS_pr = _FS_pr+PAD;
+    FL_pr = _FL_pr+PAD;
+
+    /* validate heap variables */
+    if (!s1) return NULL;
+    if (!s2B) return NULL;
+    if (!_H_pr) return NULL;
+    if (!_HM_pr) return NULL;
+    if (!_HS_pr) return NULL;
+    if (!_HL_pr) return NULL;
+    if (!_F_pr) return NULL;
+    if (!_FM_pr) return NULL;
+    if (!_FS_pr) return NULL;
+    if (!_FL_pr) return NULL;
 
     /* convert _s1 from char to int in range 0-23 */
     for (i=0; i<s1Len; ++i) {
@@ -324,10 +427,10 @@ parasail_result_t* FNAME(
                 s1[i+2],
                 s1[i+3]);
         __m256i vs2 = vNegInf;
-        const int * const restrict matrow0 = &matrix->matrix[matrix->size*s1[i+0]];
-        const int * const restrict matrow1 = &matrix->matrix[matrix->size*s1[i+1]];
-        const int * const restrict matrow2 = &matrix->matrix[matrix->size*s1[i+2]];
-        const int * const restrict matrow3 = &matrix->matrix[matrix->size*s1[i+3]];
+        const int * const restrict matrow0 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+0] : ((i+0 >= s1Len) ? s1Len-1 : i+0))];
+        const int * const restrict matrow1 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+1] : ((i+1 >= s1Len) ? s1Len-1 : i+1))];
+        const int * const restrict matrow2 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+2] : ((i+2 >= s1Len) ? s1Len-1 : i+2))];
+        const int * const restrict matrow3 = &matrix->matrix[matrix->size * ((matrix->type == PARASAIL_MATRIX_TYPE_SQUARE) ? s1[i+3] : ((i+3 >= s1Len) ? s1Len-1 : i+3))];
         __m256i vIltLimit = _mm256_cmplt_epi64_rpl(vI, vILimit);
         /* iterate over database sequence */
         for (j=0; j<s2Len+PAD; ++j) {
@@ -416,11 +519,14 @@ parasail_result_t* FNAME(
                 vES = _mm256_andnot_si256(cond, vES);
                 vEL = _mm256_andnot_si256(cond, vEL);
             }
-            vSaturationCheckMin = _mm256_min_epi64_rpl(vSaturationCheckMin, vWH);
-            vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vWH);
-            vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vWM);
-            vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vWS);
-            vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vWL);
+            /* cannot start checking sat until after J clears boundary */
+            if (j > PAD) {
+                vSaturationCheckMin = _mm256_min_epi64_rpl(vSaturationCheckMin, vWH);
+                vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vWH);
+                vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vWM);
+                vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vWS);
+                vSaturationCheckMax = _mm256_max_epi64_rpl(vSaturationCheckMax, vWL);
+            }
 #ifdef PARASAIL_TABLE
             arr_store_si256(result->stats->tables->score_table, vWH, i, s1Len, j, s2Len);
             arr_store_si256(result->stats->tables->matches_table, vWM, i, s1Len, j, s2Len);
@@ -526,15 +632,6 @@ parasail_result_t* FNAME(
     result->stats->matches = matches;
     result->stats->similar = similar;
     result->stats->length = length;
-    result->flag |= PARASAIL_FLAG_SW | PARASAIL_FLAG_DIAG
-        | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_4;
-#ifdef PARASAIL_TABLE
-    result->flag |= PARASAIL_FLAG_TABLE;
-#endif
-#ifdef PARASAIL_ROWCOL
-    result->flag |= PARASAIL_FLAG_ROWCOL;
-#endif
 
     parasail_free(_FL_pr);
     parasail_free(_FS_pr);
