@@ -110,9 +110,31 @@ parasail_result_t* FNAME(
         const char * const restrict s2, const int s2Len,
         const int open, const int gap, const parasail_matrix_t *matrix)
 {
-    parasail_profile_t *profile = parasail_profile_create_stats_neon_128_8(s1, s1Len, matrix);
-    parasail_result_t *result = INAME(profile, s2, s2Len, open, gap);
+    /* declare local variables */
+    parasail_profile_t *profile = NULL;
+    parasail_result_t *result = NULL;
+
+    /* validate inputs */
+    PARASAIL_CHECK_NULL(s2);
+    PARASAIL_CHECK_GT0(s2Len);
+    PARASAIL_CHECK_GE0(open);
+    PARASAIL_CHECK_GE0(gap);
+    PARASAIL_CHECK_NULL(matrix);
+    if (matrix->type == PARASAIL_MATRIX_TYPE_PSSM) {
+        PARASAIL_CHECK_NULL_PSSM_STATS(s1);
+    }
+    else {
+        PARASAIL_CHECK_NULL(s1);
+        PARASAIL_CHECK_GT0(s1Len);
+    }
+
+    /* initialize local variables */
+    profile = parasail_profile_create_stats_neon_128_8(s1, s1Len, matrix);
+    if (!profile) return NULL;
+    result = INAME(profile, s2, s2Len, open, gap);
+
     parasail_profile_free(profile);
+
     return result;
 }
 
@@ -121,64 +143,163 @@ STATIC parasail_result_t* PNAME(
         const char * const restrict s2, const int s2Len,
         const int open, const int gap)
 {
+    /* declare local variables */
     int32_t i = 0;
     int32_t j = 0;
     int32_t k = 0;
     int32_t end_query = 0;
     int32_t end_ref = 0;
-    const int s1Len = profile->s1Len;
-    const parasail_matrix_t *matrix = profile->matrix;
-    const int32_t segWidth = 16; /* number of values in vector unit */
-    const int32_t segLen = (s1Len + segWidth - 1) / segWidth;
-    simde__m128i* const restrict vProfile  = (simde__m128i*)profile->profile8.score;
-    simde__m128i* const restrict vProfileM = (simde__m128i*)profile->profile8.matches;
-    simde__m128i* const restrict vProfileS = (simde__m128i*)profile->profile8.similar;
-    simde__m128i* restrict pvHStore        = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* restrict pvHLoad         = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* restrict pvHMStore       = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* restrict pvHMLoad        = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* restrict pvHSStore       = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* restrict pvHSLoad        = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* restrict pvHLStore       = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* restrict pvHLLoad        = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* const restrict pvE       = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* const restrict pvEM      = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* const restrict pvES      = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* const restrict pvEL      = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* restrict pvHMax          = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* restrict pvHMMax         = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* restrict pvHSMax         = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* restrict pvHLMax         = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i vGapO = simde_mm_set1_epi8(open);
-    simde__m128i vGapE = simde_mm_set1_epi8(gap);
-    simde__m128i vZero = simde_mm_setzero_si128();
-    simde__m128i vOne = simde_mm_set1_epi8(1);
-    int8_t bias = INT8_MIN;
-    int8_t score = bias;
-    int8_t matches = bias;
-    int8_t similar = bias;
-    int8_t length = bias;
-    simde__m128i vBias = simde_mm_set1_epi8(bias);
-    simde__m128i vBias1 = simde_mm_adds_epi8(vBias,vOne);
-    simde__m128i vMaxH = vBias;
-    simde__m128i vMaxHUnit = vBias;
-    simde__m128i insert_mask = simde_mm_cmpgt_epi8(
+    int32_t s1Len = 0;
+    const parasail_matrix_t *matrix = NULL;
+    int32_t segWidth = 0;
+    int32_t segLen = 0;
+#ifdef PARASAIL_ROWCOL
+    int32_t offset = 0;
+    int32_t position = 0;
+#endif
+    simde__m128i* restrict vProfile = NULL;
+    simde__m128i* restrict vProfileM = NULL;
+    simde__m128i* restrict vProfileS = NULL;
+    simde__m128i* restrict pvHStore = NULL;
+    simde__m128i* restrict pvHLoad = NULL;
+    simde__m128i* restrict pvHMStore = NULL;
+    simde__m128i* restrict pvHMLoad = NULL;
+    simde__m128i* restrict pvHSStore = NULL;
+    simde__m128i* restrict pvHSLoad = NULL;
+    simde__m128i* restrict pvHLStore = NULL;
+    simde__m128i* restrict pvHLLoad = NULL;
+    simde__m128i* restrict pvE = NULL;
+    simde__m128i* restrict pvEM = NULL;
+    simde__m128i* restrict pvES = NULL;
+    simde__m128i* restrict pvEL = NULL;
+    simde__m128i* restrict pvHMax = NULL;
+    simde__m128i* restrict pvHMMax = NULL;
+    simde__m128i* restrict pvHSMax = NULL;
+    simde__m128i* restrict pvHLMax = NULL;
+    simde__m128i vGapO;
+    simde__m128i vGapE;
+    simde__m128i vZero;
+    simde__m128i vOne;
+    int8_t bias = 0;
+    int8_t score = 0;
+    int8_t matches = 0;
+    int8_t similar = 0;
+    int8_t length = 0;
+    simde__m128i vBias;
+    simde__m128i vBias1;
+    simde__m128i vMaxH;
+    simde__m128i vMaxHUnit;
+    simde__m128i insert_mask;
+    simde__m128i vSaturationCheckMax;
+    simde__m128i vPosLimit;
+    int8_t maxp = 0;
+    parasail_result_t *result = NULL;
+
+    /* validate inputs */
+    PARASAIL_CHECK_NULL(profile);
+    PARASAIL_CHECK_NULL(profile->profile8.score);
+    PARASAIL_CHECK_NULL(profile->matrix);
+    PARASAIL_CHECK_GT0(profile->s1Len);
+    PARASAIL_CHECK_NULL(s2);
+    PARASAIL_CHECK_GT0(s2Len);
+    PARASAIL_CHECK_GE0(open);
+    PARASAIL_CHECK_GE0(gap);
+
+    /* initialize stack variables */
+    i = 0;
+    j = 0;
+    k = 0;
+    end_query = 0;
+    end_ref = 0;
+    s1Len = profile->s1Len;
+    matrix = profile->matrix;
+    segWidth = 16; /* number of values in vector unit */
+    segLen = (s1Len + segWidth - 1) / segWidth;
+#ifdef PARASAIL_ROWCOL
+    offset = (s1Len - 1) % segLen;
+    position = (segWidth - 1) - (s1Len - 1) / segLen;
+#endif
+    vProfile  = (simde__m128i*)profile->profile8.score;
+    vProfileM = (simde__m128i*)profile->profile8.matches;
+    vProfileS = (simde__m128i*)profile->profile8.similar;
+    vGapO = simde_mm_set1_epi8(open);
+    vGapE = simde_mm_set1_epi8(gap);
+    vZero = simde_mm_setzero_si128();
+    vOne = simde_mm_set1_epi8(1);
+    bias = INT8_MIN;
+    score = bias;
+    matches = bias;
+    similar = bias;
+    length = bias;
+    vBias = simde_mm_set1_epi8(bias);
+    vBias1 = simde_mm_adds_epi8(vBias,vOne);
+    vMaxH = vBias;
+    vMaxHUnit = vBias;
+    insert_mask = simde_mm_cmpgt_epi8(
             simde_mm_set_epi8(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1),
             vZero);
-    simde__m128i vSaturationCheckMax = vBias;
-    simde__m128i vPosLimit = simde_mm_set1_epi8(INT8_MAX);
-    int8_t maxp = INT8_MAX - (int8_t)(matrix->max+1);
+    vSaturationCheckMax = vBias;
+    vPosLimit = simde_mm_set1_epi8(INT8_MAX);
+    maxp = INT8_MAX - (int8_t)(matrix->max+1);
+
+    /* initialize result */
 #ifdef PARASAIL_TABLE
-    parasail_result_t *result = parasail_result_new_table3(segLen*segWidth, s2Len);
+    result = parasail_result_new_table3(segLen*segWidth, s2Len);
 #else
 #ifdef PARASAIL_ROWCOL
-    parasail_result_t *result = parasail_result_new_rowcol3(segLen*segWidth, s2Len);
-    const int32_t offset = (s1Len - 1) % segLen;
-    const int32_t position = (segWidth - 1) - (s1Len - 1) / segLen;
+    result = parasail_result_new_rowcol3(segLen*segWidth, s2Len);
 #else
-    parasail_result_t *result = parasail_result_new_stats();
+    result = parasail_result_new_stats();
 #endif
 #endif
+    if (!result) return NULL;
+
+    /* set known flags */
+    result->flag |= PARASAIL_FLAG_SW | PARASAIL_FLAG_STRIPED
+        | PARASAIL_FLAG_STATS
+        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_16;
+#ifdef PARASAIL_TABLE
+    result->flag |= PARASAIL_FLAG_TABLE;
+#endif
+#ifdef PARASAIL_ROWCOL
+    result->flag |= PARASAIL_FLAG_ROWCOL;
+#endif
+
+    /* initialize heap variables */
+    pvHStore  = parasail_memalign_simde__m128i(16, segLen);
+    pvHLoad   = parasail_memalign_simde__m128i(16, segLen);
+    pvHMStore = parasail_memalign_simde__m128i(16, segLen);
+    pvHMLoad  = parasail_memalign_simde__m128i(16, segLen);
+    pvHSStore = parasail_memalign_simde__m128i(16, segLen);
+    pvHSLoad  = parasail_memalign_simde__m128i(16, segLen);
+    pvHLStore = parasail_memalign_simde__m128i(16, segLen);
+    pvHLLoad  = parasail_memalign_simde__m128i(16, segLen);
+    pvE       = parasail_memalign_simde__m128i(16, segLen);
+    pvEM      = parasail_memalign_simde__m128i(16, segLen);
+    pvES      = parasail_memalign_simde__m128i(16, segLen);
+    pvEL      = parasail_memalign_simde__m128i(16, segLen);
+    pvHMax    = parasail_memalign_simde__m128i(16, segLen);
+    pvHMMax   = parasail_memalign_simde__m128i(16, segLen);
+    pvHSMax   = parasail_memalign_simde__m128i(16, segLen);
+    pvHLMax   = parasail_memalign_simde__m128i(16, segLen);
+
+    /* validate heap variables */
+    if (!pvHStore) return NULL;
+    if (!pvHLoad) return NULL;
+    if (!pvHMStore) return NULL;
+    if (!pvHMLoad) return NULL;
+    if (!pvHSStore) return NULL;
+    if (!pvHSLoad) return NULL;
+    if (!pvHLStore) return NULL;
+    if (!pvHLLoad) return NULL;
+    if (!pvE) return NULL;
+    if (!pvEM) return NULL;
+    if (!pvES) return NULL;
+    if (!pvEL) return NULL;
+    if (!pvHMax) return NULL;
+    if (!pvHMMax) return NULL;
+    if (!pvHSMax) return NULL;
+    if (!pvHLMax) return NULL;
 
     parasail_memset_simde__m128i(pvHStore, vBias, segLen);
     parasail_memset_simde__m128i(pvHMStore, vBias, segLen);
@@ -523,15 +644,6 @@ end:
     result->stats->matches = matches - bias;
     result->stats->similar = similar - bias;
     result->stats->length = length - bias;
-    result->flag |= PARASAIL_FLAG_SW | PARASAIL_FLAG_STRIPED
-        | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_16;
-#ifdef PARASAIL_TABLE
-    result->flag |= PARASAIL_FLAG_TABLE;
-#endif
-#ifdef PARASAIL_ROWCOL
-    result->flag |= PARASAIL_FLAG_ROWCOL;
-#endif
 
     parasail_free(pvHLMax);
     parasail_free(pvHSMax);
